# %%
try:
    from numba.experimental import jitclass
except:
    from numba import jitclass
from numba import njit, types, vectorize, prange
from numba import int32, float64 # import the types
from pylab import np, plt
plt.style.use('ggplot')


# %%
@jitclass(
    spec=[
        # name plate data
        ('npp',   int32),
        ('npp_inv', float64),
        ('IN',  float64),
        # electrical parameters
        ('R',   float64),
        ('Ld',  float64),
        ('Lq',  float64),
        ('KE',  float64),
        # mechanical parameters
        ('Js',  float64),
        ('Js_inv', float64),
        ('B',   float64),
        # states
        ('NS',    int32),
        ('x',   float64[:]),
        ('Tem', float64),
        # inputs
        ('uab',   float64[:]),
        ('udq',   float64[:]),
        ('TLoad', float64),
        # output
        ('cosT', float64),
        ('sinT', float64),
        ('theta_d', float64),
        ('theta_mech', float64),
        ('omega_elec', float64),
        ('iab', float64[:]),
    ])
class The_AC_Machines:
    def __init__(self):
        # name plate data
        self.npp = 4
        self.npp_inv = 1.0/self.npp
        self.IN = 3 # Arms (line-to-line)
        # electrical parameters
        self.R = 1.1
        self.Ld = 5e-3
        self.Lq = 5e-3
        self.KE = 0.095
        # mechanical parameters
        self.Js = 0.0006168  # kg.m^2
        self.Js_inv = 1.0/self.Js
        self.B  = 0*0.7e-3 # Nm.s
        # states
        self.NS = 5
        self.x = np.zeros(self.NS, dtype=np.float64)
        self.Tem = 0.0
        # inputs
        self.uab = np.zeros(2, dtype=np.float64)
        self.udq = np.zeros(2, dtype=np.float64)
        self.TLoad = 0
        # output
        self.cosT = 0.0
        self.sinT = 0.0
        self.theta_d = 0.0
        self.theta_mech = 0.0
        self.omega_elec = 0.0
        self.iab = np.zeros(2, dtype=np.float64)

@jitclass(
    spec=[
        # constants
        ('CL_TS', float64),
        ('VL_TS', float64),
        # feedback / input
        ('theta_d', float64),
        ('omega_elec', float64),
        ('iab', float64[:]),
        # states
        ('timebase', float64),
        ('cosT', float64),
        ('sinT', float64),
        ('idq', float64[:]),
        # commands
        ('idq_cmd', float64[:]),
        ('udq_cmd', float64[:]),
        ('uab_cmd', float64[:]),
        ('cmd_rpm_speed', float64),
        # MOTOR
            # name plate data
            ('npp',   int32),
            ('IN',  float64),
            # electrical parameters
            ('R',   float64),
            ('Ld',  float64),
            ('Lq',  float64),
            ('KE',  float64),
            # mechanical parameters
            ('Js',  float64),
    ])
class The_Motor_Controller:
    def __init__(self, CL_TS, VL_TS, human):
        # constants
        self.CL_TS = CL_TS
        self.VL_TS = VL_TS
        # feedback / input
        self.theta_d = 0.0
        self.omega_elec = 0.0
        self.iab = np.zeros(2, dtype=np.float64)
        # states
        self.timebase = 0.0
        self.cosT = 0.0
        self.sinT = 0.0
        self.idq = np.zeros(2, dtype=np.float64)
        # commands 
        self.idq_cmd = np.zeros(2, dtype=np.float64)
        self.udq_cmd = np.zeros(2, dtype=np.float64)
        self.uab_cmd = np.zeros(2, dtype=np.float64)
        self.cmd_rpm_speed = 0.0;
        # MOTOR
        self.npp = human.npp
        self.IN  = human.IN
        self.R   = human.R
        self.Ld  = human.Ld
        self.Lq  = human.Lq
        self.KE  = human.KE
        self.Js  = human.Js

@jitclass(
    spec=[
        # feedback / inputs
        ('iab_prev', float64[:]),
        ('iab_curr', float64[:]),
        ('uab_prev', float64[:]),
        # states
        ('cosT', float64),
        ('sinT', float64),
        # outputs
        ('theta_d', float64),
        ('omega_elec', float64),
        # MOTOR
            # name plate data
            ('npp',   int32),
            ('IN',  float64),
            # electrical parameters
            ('R',   float64),
            ('Ld',  float64),
            ('Lq',  float64),
            ('KE',  float64),
            # mechanical parameters
            ('Js',  float64),
    ])
class The_Observer:
    def __init__(self, human):
        # feedback / input
        self.theta_d = 0.0
        self.omega_elec = 0.0
        self.iab = np.zeros(2, dtype=np.float64)
        # states
        self.timebase = 0.0
        self.cosT = 0.0
        self.sinT = 0.0
        self.idq = np.zeros(2, dtype=np.float64)
        # commands 
        self.idq_cmd = np.zeros(2, dtype=np.float64)
        self.udq_cmd = np.zeros(2, dtype=np.float64)
        self.uab_cmd = np.zeros(2, dtype=np.float64)
        self.cmd_rpm_speed = 0.0;
        # MOTOR
        self.npp = human.npp
        self.IN  = human.IN
        self.R   = human.R
        self.Ld  = human.Ld
        self.Lq  = human.Lq
        self.KE  = human.KE
        self.Js  = human.Js


@jitclass(
    spec=[
        ('Kp', float64),
        ('Ki', float64),
        ('Err', float64),
        ('Ref', float64),
        ('Fbk', float64),
        ('Out', float64),
        ('OutLimit', float64),
        ('ErrPrev', float64),
        ('OutPrev', float64),
    ])
class The_PI_Regulator:
    def __init__(self, KP_CODE, KI_CODE, OUTPUT_LIMIT):
        self.Kp = KP_CODE
        self.Ki = KI_CODE
        self.Err      = 0.0
        self.Ref      = 0.0
        self.Fbk      = 0.0
        self.Out      = 0.0
        self.OutLimit = OUTPUT_LIMIT
        self.ErrPrev  = 0.0
        self.OutPrev  = 0.0

@jitclass(
    spec=[
        ('sensorless_position', int32),
        ('sensorless_speed', int32),
        # MOTOR
            # name plate data
            ('npp',   int32),
            ('IN',  float64),
            # electrical parameters
            ('R',   float64),
            ('Ld',  float64),
            ('Lq',  float64),
            ('KE',  float64),
            # mechanical parameters
            ('Js',  float64),
    ])
class The_Human:
    def __init__(self):
        self.sensorless_position = False
        self.sensorless_speed    = False
        # name plate data
        self.npp = 4
        self.IN = 3 # Arms (line-to-line)
        # electrical parameters
        self.R = 1.1
        self.Ld = 5e-3
        self.Lq = 5e-3
        self.KE = 0.095
        # mechanical parameters
        self.Js = 0.0006168  # kg.m^2



@njit(nogil=True)
def MACHINE_DYNAMICS(t, x, ACM, CLARKE_TRANS_TORQUE_GAIN=1.5):
    fx = np.zeros(5)

    # 电磁子系统 (id, iq)
    fx[0] = (ACM.udq[0] - ACM.R * x[0] + x[2]*ACM.Lq*x[1]) / ACM.Ld
    fx[1] = (ACM.udq[1] - ACM.R * x[1] - x[2]*ACM.Ld*x[0] - x[2]*ACM.KE) / ACM.Lq

    # 机械子系统 (omega_r_elec, theta_d_elec, theta_r_mech)
    ACM.Tem = CLARKE_TRANS_TORQUE_GAIN * ACM.npp * \
        (x[1]*ACM.KE + (ACM.Ld - ACM.Lq)*x[0]*x[1]) # 电磁转矩 Tem 计算
    fx[2] = (ACM.Tem - ACM.TLoad - ACM.B * x[2]) * ACM.npp/ACM.Js # elec. angular rotor speed
    fx[3] = x[2]              # elec. angular rotor position(bounded)
    fx[4] = x[2]/ACM.npp  # mech. angular rotor position(accumulated)
    return fx

@njit(nogil=True)
def RK4(t, ACM, hs): # 四阶龙格库塔法
    NS = ACM.NS
    k1, k2, k3, k4 = np.zeros(NS), np.zeros(NS), np.zeros(NS), np.zeros(NS)
    xk, fx = np.zeros(NS), np.zeros(NS)

    if False:
        """ this is about twice slower than loop through the element one by one """ 
        fx = MACHINE_DYNAMICS(t, ACM.x, ACM) # @timer.t,
        k1 = fx * hs
        xk = ACM.x + k1*0.5

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # @timer.t+hs/2.,
        k2 = fx * hs
        xk = ACM.x + k2*0.5

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # @timer.t+hs/2.,
        k3 = fx * hs
        xk = ACM.x + k3

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # timer.t+hs,
        k4 = fx * hs
        ACM.x = ACM.x + (k1 + 2*(k2 + k3) + k4)/6.0
    else:
        for i in range(NS):
            k1[i] = fx[i] * hs
            xk[i] = ACM.x[i] + k1[i]*0.5

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # @timer.t+hs/2.,
        for i in range(NS):
            k2[i] = fx[i] * hs
            xk[i] = ACM.x[i] + k2[i]*0.5

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # @timer.t+hs/2.,
        for i in range(NS):
            k3[i] = fx[i] * hs
            xk[i] = ACM.x[i] + k3[i]

        fx = MACHINE_DYNAMICS(t, xk, ACM)  # timer.t+hs,
        for i in range(NS):
            k4[i] = fx[i] * hs
            ACM.x[i] = ACM.x[i] + (k1[i] + 2*(k2[i] + k3[i]) + k4[i])/6.0
            # derivatives
            # ACM.x_dot[i] = (k1[i] + 2*(k2[i] + k3[i]) + k4[i])/6.0 / hs 

@njit(nogil=True)
def incremental_pi(reg):
    reg.Err = reg.Ref - reg.Fbk
    reg.Out = reg.OutPrev + \
        reg.Kp * (reg.Err - reg.ErrPrev) + \
        reg.Ki * reg.Err
    if reg.Out >    reg.OutLimit:
        reg.Out =   reg.OutLimit
    elif reg.Out < -reg.OutLimit:
        reg.Out =  -reg.OutLimit
    reg.ErrPrev = reg.Err
    reg.OutPrev = reg.Out

@njit(nogil=True)
def FOC(CTRL, reg_id, reg_iq, reg_speed):
    reg_speed.Ref = CTRL.cmd_rpm_speed / 60 * 2*np.pi * CTRL.npp
    reg_speed.Fbk = CTRL.omega_elec
    incremental_pi(reg_speed)

    reg_id.Ref = 0.0
    reg_id.Fbk = CTRL.idq[0]
    incremental_pi(reg_id)
    CTRL.udq_cmd[0] = reg_id.Out

    reg_iq.Ref = reg_speed.Out
    reg_iq.Fbk = CTRL.idq[1]
    incremental_pi(reg_iq)
    CTRL.udq_cmd[1] = reg_iq.Out

    return CTRL.udq_cmd



""" DSP """
@njit(nogil=True)
def DSP(ACM, CTRL, reg_id, reg_iq, reg_speed, human):
    CTRL.timebase += CTRL.CL_TS

    """ Measurment """
    CTRL.iab[0]     = ACM.iab[0] # + offset, scale error , noise
    CTRL.iab[1]     = ACM.iab[1] # + offset, scale error , noise
    if human.sensorless_speed:
        CTRL.omega_elec = ACM.omega_elec # OB.omega_elec
    else:
        CTRL.omega_elec = ACM.omega_elec
    if human.sensorless_position:
        CTRL.theta_d = ACM.theta_d # OB.theta_d
    else:
        CTRL.theta_d = ACM.theta_d
    # do this once per control interrupt
    CTRL.cosT = np.cos(CTRL.theta_d)
    CTRL.sinT = np.sin(CTRL.theta_d)
    # Park transformation
    CTRL.idq[0] = CTRL.iab[0] * CTRL.cosT + CTRL.iab[1] * CTRL.sinT
    CTRL.idq[1] = CTRL.iab[0] *-CTRL.sinT + CTRL.iab[1] * CTRL.cosT

    """ Controller """
    CTRL.udq_cmd = FOC(CTRL, reg_id, reg_iq, reg_speed)
    # Inverse Park transformation
    CTRL.uab_cmd[0] = CTRL.udq_cmd[0] * CTRL.cosT + CTRL.udq_cmd[1] *-CTRL.sinT
    CTRL.uab_cmd[1] = CTRL.udq_cmd[0] * CTRL.sinT + CTRL.udq_cmd[1] * CTRL.cosT

    """ Inverter """
    ACM.uab[0] = CTRL.uab_cmd[0] # + current denpendent distorted voltage
    ACM.uab[1] = CTRL.uab_cmd[1] # + current denpendent distorted voltage

""" MAIN for Real-time simulation """
@njit(nogil=True)
def ACMSimPyIncremental(parameters,
            t0, TIME,
            ACM=None,
            CTRL=None,
            reg_id=None,
            reg_iq=None,
            reg_speed=None,
            human=None,
            ):
    MACHINE_TS = CTRL.CL_TS
    down_sampling_ceiling = int(CTRL.CL_TS / MACHINE_TS); #print('\tdown sample:', down_sampling_ceiling)

    # watch variabels
    machine_times  = np.arange(t0, t0+TIME, MACHINE_TS)
    control_times  = np.arange(t0, t0+TIME, CTRL.CL_TS)
    id = np.zeros_like(control_times)
    iq = np.zeros_like(control_times)
    ia = np.zeros_like(control_times)
    ib = np.zeros_like(control_times)
    speed = np.zeros_like(control_times)

    # Main loop
    # print('\tt0 =', t0)
    jj = 0; watch_index = 0
    for ii in range(len(machine_times)):

        t = machine_times[ii]

        """ Machine Simulation @ MACHINE_TS """
        # Park transformation
        ACM.udq[0] = ACM.uab[0] *  ACM.cosT + ACM.uab[1] * ACM.sinT
        ACM.udq[1] = ACM.uab[0] * -ACM.sinT + ACM.uab[1] * ACM.cosT
        # Numerical Integration (ode4) with 5 states
        RK4(t, ACM, hs=MACHINE_TS)

        """ Machine Simulation Output @ MACHINE_TS """
        # Generate output variables for easy access
        ACM.omega_elec  = ACM.x[2]
        ACM.theta_d     = ACM.x[3]
        ACM.theta_mech  = ACM.x[4]
        # Inverse Park transformation
        ACM.cosT = np.cos(ACM.theta_d)
        ACM.sinT = np.sin(ACM.theta_d)
        ACM.iab[0] = ACM.x[0] * ACM.cosT + ACM.x[1] *-ACM.sinT
        ACM.iab[1] = ACM.x[0] * ACM.sinT + ACM.x[1] * ACM.cosT

        jj += 1
        if jj >= down_sampling_ceiling:
            jj = 0

            """ DSP @ CL_TS """
            DSP(ACM=ACM,
                CTRL=CTRL,
                reg_id=reg_id,
                reg_iq=reg_iq,
                reg_speed=reg_speed,
                human=human)

            """ Console @ CL_TS """
            if t < 1.0:
                CTRL.cmd_rpm_speed = 50
            elif t < 1.5:
                ACM.TLoad = 2
            elif t < 2.0:
                CTRL.cmd_rpm_speed = 200
            elif t < 3.0:
                CTRL.cmd_rpm_speed = -200
            elif t < 4.0:
                CTRL.cmd_rpm_speed = 0
            elif t < 4.5:
                CTRL.cmd_rpm_speed = 2000
            # else: # don't implement else to allow command from IPython console

            """ Watch @ CL_TS """
            id[watch_index] = ACM.x[0]
            iq[watch_index] = ACM.x[1]
            ia[watch_index] = CTRL.iab[0]
            ib[watch_index] = CTRL.iab[1]
            speed[watch_index] = CTRL.omega_elec / (2*np.pi*ACM.npp) * 60
            watch_index += 1

    return control_times, id, iq, ia, ib, speed


# %% Test incremental simulation
if __name__ == '__main__':

    # Simulation Basics
    CL_TS      = 1e-4
    TIME_SLICE = 1.0

    # Human
    human = The_Human()

    # init
    ACM       = The_AC_Machines()
    CTRL      = The_Motor_Controller(CL_TS, 5*CL_TS, human)
    reg_id    = The_PI_Regulator(6.39955, 6.39955*237.845*CTRL.CL_TS, 600)
    reg_iq    = The_PI_Regulator(6.39955, 6.39955*237.845*CTRL.CL_TS, 600)
    reg_speed = The_PI_Regulator(0.0380362, 0.0380362*30.5565*CTRL.VL_TS, 1*1.414*ACM.IN)

    # Global arrays
    global_speed = None

    for ii in range(0, 10):
        """perform animation step"""
        control_times, id, iq, ia, ib, speed, = \
            ACMSimPyIncremental(1,
                        t0=ii*TIME_SLICE, TIME=TIME_SLICE, 
                        ACM=ACM,
                        CTRL=CTRL,
                        reg_id=reg_id,
                        reg_iq=reg_iq,
                        reg_speed=reg_speed,
                        human=human)
        if global_speed is None:
            global_speed = speed
        else:
            global_speed = np.append(global_speed, speed)
        print(len(speed), end='|')
        print(max(speed), end='|')
        print(len(global_speed), end='|')
        print(max(global_speed))

# %%



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# DEFAULT PACKAGES
# ///////////////////////////////////////////////////////////////
import os

# IMPORT / GUI, SETTINGS AND WIDGETS
# ///////////////////////////////////////////////////////////////
# Packages
from datetime import datetime
# from app.packages.pyside_or_pyqt import * # Qt
# from PySide2.QtWidgets import *
from app_modules import * # Settings


# GLOBALS
send_by = None

# MAIN WINDOW
# ///////////////////////////////////////////////////////////////
class Message(QWidget):
    def __init__(self, message, me_send):
        QWidget.__init__(self)
        global send_by
        send_by = me_send

        self.setMinimumHeight(20)
        self.setup_ui()
        self.setFixedHeight(self.layout.sizeHint().height())

        # SET MESSAGE
        self.message.setText(message)

        # SET DATE TIME
        date_time = datetime.now()
        date_time_format = date_time.strftime("%m/%d/%Y %H:%M")
        self.data_message.setText(str(date_time_format))

    def setup_ui(self):
        # LAYOUT
        self.layout = QHBoxLayout(self)
        self.layout.setContentsMargins(0,0,0,0)

        # FRAME BG
        self.bg = QFrame()
        if send_by:
            self.bg.setStyleSheet("#bg {background-color: #0e0e0f; border-radius: 10px; margin-left: 150px; } #bg:hover { background-color: #252628; }")
        else:
            self.bg.setStyleSheet("#bg {background-color: #28282b; border-radius: 10px; margin-right: 150px; } #bg:hover { background-color: #252628; }")
        self.bg.setObjectName("bg")

        # FRAME BG
        self.btn = QPushButton()
        self.btn.setMinimumSize(40, 40)
        self.btn.setMaximumSize(40, 40)
        self.btn.setStyleSheet("""
        QPushButton {
            background-color: transparent;
            border-radius: 20px;
            background-repeat: no-repeat;
            background-position: center;
            background-image: url(:/icons_svg/images/icons_svg/icon_more_options.svg);
        }
        QPushButton:hover {
            background-color: rgb(61, 62, 65);
        }
        QPushButton:pressed {
            background-color: rgb(16, 17, 18);
        }        
        """)

        if send_by:
            self.layout.addWidget(self.bg)
            self.layout.addWidget(self.btn)
        else:
            self.layout.addWidget(self.btn)
            self.layout.addWidget(self.bg)

        # LAYOUT INSIDE
        self.layout_inside = QVBoxLayout(self.bg)
        self.layout.setContentsMargins(10,10,10,10)

        # LABEL MESSAGE 
        self.message = QLabel()
        self.message.setText("message test")
        self.message.setStyleSheet("font: 500 11pt 'Segoe UI'")
        self.message.setTextInteractionFlags(Qt.TextSelectableByMouse|Qt.TextSelectableByKeyboard)

        # LABEL MESSAGE 
        self.data_message = QLabel()
        self.data_message.setText("date")
        self.data_message.setStyleSheet("font: 8pt 'Segoe UI'; color: #4c5154")
        if send_by:
            self.data_message.setAlignment(Qt.AlignRight)
        else:
            self.data_message.setAlignment(Qt.AlignLeft)

        self.layout_inside.addWidget(self.message)
        self.layout_inside.addWidget(self.data_message)



# coding:u8
# this allows to use "from utils import conversion" in __main__.py
import os, sys

# print(
#     os.path.dirname(os.path.realpath(__file__)),
#     os.path.dirname(os.path.realpath(__file__))+'/guiv2',
#     os.path.dirname(os.path.realpath(__file__))+'/utils',
#     os.path.dirname(os.path.realpath(__file__))+'/jsons',
#     os.path.dirname(os.path.realpath(__file__))+'/acmdesignv2',
#     os.path.dirname(os.path.realpath(__file__))+'/acmsimpyv1',
#     sep='\n'
# )

# 为了不需要手动修改 mainWindow.py 中的相对路径的 import 
# https://stackoverflow.com/questions/16981921/relative-imports-in-python-3
sys.path.append( os.path.dirname(os.path.realpath(__file__))                )
sys.path.append( os.path.dirname(os.path.realpath(__file__))+'/guiv2'       )
sys.path.append( os.path.dirname(os.path.realpath(__file__))+'/utils'       )
sys.path.append( os.path.dirname(os.path.realpath(__file__))+'/jsons'       )
sys.path.append( os.path.dirname(os.path.realpath(__file__))+'/acmdesignv2' )
sys.path.append( os.path.dirname(os.path.realpath(__file__))+'/acmsimpyv1'  )

__version__ = '1.2.3'
__description__ = 'A GUI for analysis of electric machinery'
__author__ = 'Jiahao Chen'
__author_email__ = 'horychen@qq.com'
__url__ = 'https://github.com/horychen/emachinery'


# import emachinery.gui.main
import emachinery.acmsimcv5.CJHAcademicPlot.__cjhAcademicPlotSettings
# import emachinery.acmsimcv5.CopySimToExp



################################################################################
##
## BY: WANDERSON M.PIMENTA
## PROJECT MADE WITH: Qt Designer and PySide2
## V: 1.0.0
##
## This project can be used freely for all uses, as long as they maintain the
## respective credits only in the Python scripts, any information in the visual
## interface (GUI) can be modified without any implication.
##
## There are limitations on Qt licenses if you want to use your products
## commercially, I recommend reading them on the official website:
## https://doc.qt.io/qtforpython/licenses.html
##
################################################################################

class Style():

    style_bt_standard = (
    """
    QPushButton {
        background-image: ICON_REPLACE;
        background-position: left center;
        background-repeat: no-repeat;
        border: none;
        border-left: 28px solid rgb(27, 29, 35);
        background-color: rgb(27, 29, 35);
        text-align: left;
        padding-left: 45px;
    }
    QPushButton[Active=true] {
        background-image: ICON_REPLACE;
        background-position: left center;
        background-repeat: no-repeat;
        border: none;
        border-left: 28px solid rgb(27, 29, 35);
        border-right: 5px solid rgb(44, 49, 60);
        background-color: rgb(27, 29, 35);
        text-align: left;
        padding-left: 45px;
    }
    QPushButton:hover {
        background-color: rgb(63, 4, 70); /*rgb(33, 37, 43);*/
        border-left: 28px solid rgb(63, 4, 70);
    }
    QPushButton:pressed {
        background-color: rgb(85, 170, 255);
        border-left: 28px solid rgb(85, 170, 255);
    }
    """
    )

    style_custom = (
    """
    QPushButton {
        border: 2px solid rgb(52, 59, 72);
        border-radius: 5px; 
        background-color: rgb(52, 59, 72);
    }
    QPushButton:hover {
        background-color: rgb(57, 65, 80);
        border: 2px solid rgb(61, 70, 86);
    }
    QPushButton:pressed {   
        background-color: rgb(35, 40, 49);
        border: 2px solid rgb(43, 50, 61);
    }
    """
    )

    myGroupBoxStyle = '''
    QGroupBox {
        /*border: None;              /*<-----  None              */
        border: 1px solid #76797C;   /*<-----  1px solid #76797C */
        /* border-radius: 2px; */
        /* margin-top: 20px;*/
    }
    /* https://stackoverflow.com/questions/54408723/how-to-build-a-flat-groupbox-in-pyqt5
    QGroupBox::title {
        subcontrol-origin: margin;
        subcontrol-position: top center;
        padding-left: 10px;
        padding-right: 10px;
        padding-top: 10px;
    }*/
    '''
    # app.setStyleSheet(myGroupBoxStyle)



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT
# ///////////////////////////////////////////////////////////////
# Packages
# from app.packages.pyside_or_pyqt import *
# from PySide2.QtWidgets import *
# from PySide2.QtGui import QPainterPath, QPen, QImage
from app_modules import * # Settings
# Modules
# import os

# FRIEND MENU MESSAGE / MESSAGE BUTTON
# Friends messages with name and status
class FriendMessageButton(QWidget):
    # SIGNALS
    # ///////////////////////////////////////////////////////////////
    clicked = Signal()
    released = Signal()

    def __init__(
        self,
        id,
        user_image,
        user_name,
        user_descrition,
        user_status,
        unread_messages,
        is_active
    ):
        QWidget.__init__(self)

        # ICON PATH
        # ///////////////////////////////////////////////////////////////
        image = user_image
        app_path = os.path.abspath(os.getcwd())
        image_path = os.path.join(app_path, image)

        # CUSTOM PARAMETERS
        # ///////////////////////////////////////////////////////////////
        self.user_image = image_path
        self.user_name = user_name
        self.user_description = user_descrition
        self.user_status = user_status
        self.unread_messages = unread_messages
        self.is_active = is_active
        self._status_color = "#46b946"
        self.bg_color_entered = "#22CCCCCC"
        self.bg_color_leave = "#00000000"
        self.bg_color_active = "#33CCCCCC"
        self._bg_color = self.bg_color_leave      


        # SETUP
        self.setFixedSize(240, 50)
        self.setCursor(Qt.PointingHandCursor)
        self.setObjectName(str(id))
        self.setup_ui()

        # SHOW UNREAD MESSAGES
        if self.unread_messages > 0:
            self.label_messages.show()
            self.label_messages.setText(str(self.unread_messages))
        
        # CHANGE COLOR
        if self.user_status == "online":
            self._status_color = "#46b946"
        elif self.user_status == "ilde":
            self._status_color = "#ff9955"
        elif self.user_status == "busy":
            self._status_color = "#a02c2c"
        elif self.user_status == "invisible":
            self._status_color = "#808080"
        
        # CHANGE OPACITY
        if self.user_status == "invisible":
            self.opacity = QGraphicsOpacityEffect()
            self.opacity.setOpacity(0.4)
            self.setGraphicsEffect(self.opacity)

    def reset_unread_message(self):
        self.unread_messages = 0
        self.label_messages.hide()
        self.repaint()

    # MOUSE PRESS
    # Event triggered when the left button is pressed
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            # EMIT SIGNAL
            self.clicked.emit()

    # MOUSE RELEASE
    # Event fired when the mouse leaves the BTN
    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.released.emit()

    # MOUSE ENTER
    # Event fired when the mouse enter
    def enterEvent(self, event):
        if not self.is_active:
            self._bg_color = self.bg_color_entered
            self.repaint()

    # MOUSE LEAVE
    # Event fired when the mouse leave
    def leaveEvent(self, event):
        if not self.is_active:
            self._bg_color = self.bg_color_leave
            self.repaint()

    def set_active(self, active):
        if active:
            self.is_active = active
        else:
            self.is_active = active 
            self._bg_color = self.bg_color_leave
        self.repaint()

    # SETUP WIDGETS
    # ///////////////////////////////////////////////////////////////
    def setup_ui(self):
        # FRAME TEXT
        self.text_frame = QFrame(self)
        self.text_frame.setGeometry(60, 0, 170, 50)

        # USER NAME
        self.label_user = QLabel(self.text_frame)
        self.label_user.setGeometry(0, 8, self.text_frame.width(), 20)
        self.label_user.setAlignment(Qt.AlignVCenter)        
        self.label_user.setText(self.user_name.capitalize())
        self.label_user.setStyleSheet("color: #e7e7e7; font: 700 10pt 'Segoe UI';")

        # USER STATUS
        self.label_description = QLabel(self.text_frame)
        self.label_description.setGeometry(0, 22, self.text_frame.width(), 18)
        self.label_description.setAlignment(Qt.AlignVCenter)        
        self.label_description.setText(self.user_description)
        self.label_description.setStyleSheet("color: #A6A6A6; font: 9pt 'Segoe UI';")

        # USER STATUS
        self.label_messages = QLabel(self)
        self.label_messages.setFixedSize(35, 20)
        self.label_messages.setAlignment(Qt.AlignCenter)
        self.label_messages.setStyleSheet("""
            background-color: #1e2021;
            padding-left: 5px;
            padding-right: 5px;
            color: #bdff00;
            border-radius: 10px;
            border: 3px solid #333;
            font: 9pt 'Segoe UI';
        """)
        self.label_messages.move(self.width() - 45, 16)
        self.label_messages.hide()

    # PAINT EVENT
    # PAINT USER IMAGE EVENTS
    # ///////////////////////////////////////////////////////////////
    def paintEvent(self, event):
        # PAINTER USER IMAGE
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setPen(Qt.NoPen)

        # RECT
        rect = QRect(10, 5, 40, 40)

        # DRAW BG
        if self.is_active:
            painter.setBrush(QBrush(QColor(self.bg_color_active)))            
        else:
            painter.setBrush(QBrush(QColor(self._bg_color)))
        painter.drawRoundedRect(5, 0, 230, 50, 25, 25)

        # CIRCLE        
        painter.setBrush(QBrush(QColor("#000000")))
        painter.drawEllipse(rect)

        # DRAW USER IMAGE
        self.draw_user_image(painter, self.user_image, rect)        

        painter.end()

        # DRAW USER IMAGE
        if self.user_status != "invisible":
            self.draw_status(self.user_image, rect)

    # DRAW USER IMAGE
    # ///////////////////////////////////////////////////////////////
    def draw_user_image(self, qp, image, rect):
        user_image = QImage(image)
        path = QPainterPath()
        path.addEllipse(rect)
        qp.setClipPath(path)
        qp.drawImage(rect, user_image)

    # DRAW STATUS
    # ///////////////////////////////////////////////////////////////
    def draw_status(self, status, rect):
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)

        # PEN
        pen = QPen()
        pen.setWidth(3)
        pen.setColor(QColor("#151617"))
        painter.setPen(pen)

        # BRUSH/STATUS COLOR
        painter.setBrush(QBrush(QColor(self._status_color)))

        # DRAW
        painter.drawEllipse(rect.x() + 27, rect.y() + 27, 13, 13)
        painter.end()
        


if __name__ == '__main__':
    import os, sys; sys.path.insert(1, '../../CJHAcademicPlot'); 
    from __cjhAcademicPlotSettings import CJHStylePlot
    csp = CJHStylePlot(path2info_dot_dat=__file__, nrows=7)
    csp.load()
    """ Plot Menu
    Load Torque [Nm]
	Speed [rpm]
	Offset [V]
	Flux Alpha Error [Wb]
	Flux Alpha [Wb]
	Flux Ctrl Error [Wb]
	Speed Ctrl Error [rpm]
	['ACM.TLoad', 'marino.xTL', 'marino.xOmg*ELEC_RAD_PER_SEC_2_RPM', 'ACM.rpm', 'ACM.rpm_cmd', 'watch.offset[0]', 'watch.offset_compensation[0]', 'watch.offset[1]', 'watch.offset_compensation[1]', 'ACM.psi_Amu - watch.psi_2[0]', 'ACM.psi_Amu', 'watch.psi_2[0]', 'marino.e_psi_Dmu', 'marino.e_psi_Qmu', 'ACM.rpm - marino.xOmg*ELEC_RAD_PER_SEC_2_RPM']
	"""


    list_of_ylabel = [    "Load Torque [Nm]",
    "Speed [rpm]",
    "Offset [V]",
    "Flux Alpha Error [Wb]",
    "Flux Alpha [Wb]",
    "Flux Ctrl Error [Wb]",
    "Speed Ctrl Error [rpm]",
    ]

    list_of_signalNames = [
    	######################################
        ( 
			"ACM.TLoad",
			"marino.xTL",
        ),
        	######################################
        ( 
			"marino.xOmg*ELEC_RAD_PER_SEC_2_RPM",
			"ACM.rpm",
			"ACM.rpm_cmd",
        ),
        	######################################
        ( 
			"watch.offset[0]",
			"watch.offset_compensation[0]",
			"watch.offset[1]",
			"watch.offset_compensation[1]",
        ),
        	######################################
        ( 
			"ACM.psi_Amu - watch.psi_2[0]",
        ),
        	######################################
        ( 
			"ACM.psi_Amu",
			"watch.psi_2[0]",
        ),
        	######################################
        ( 
			"marino.e_psi_Dmu",
			"marino.e_psi_Qmu",
        ),
        	######################################
        ( 
			"ACM.rpm - marino.xOmg*ELEC_RAD_PER_SEC_2_RPM",
        ),
        ]

    for index in range(5):
        csp.plot(list_of_ylabel, list_of_signalNames, index=index)


# from gui.main import main

# from emachinery.guiv2.main import startApp as main

import os
def main():
    os.system('python '+os.path.dirname(__file__) + '/guiv2/main.py')

if __name__ == '__main__':
    main()
    # 这里不要写任何其他代码了，emy只会调用本文件中import进来的main()函数



print('RUN main.py')

import sys
print('\tDebug sys.path:')
for el in (sys.path):
    print('\t', el)
from app_modules import *

class emyMainWindow(QMainWindow):
    def __init__(self, ui=None):
        QMainWindow.__init__(self) # QMainWindow.__init__(self, None, Qt.WindowStayclssOnTopHint | Qt.FramelessWindowHint) # always on top

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        ## PRINT ==> SYSTEM
            # 📕: error message
            # 📙: warning message
            # 📗: ok status message
            # 📘: action message
            # 📓: canceled status message
            # 📔: Or anything you like and want to recognize immediately by color
        print('\t📗 [yellow]System[/]: ' + platform.system())
        print('\t📗 [yellow]Version[/]: '+ platform.release())

        ########################################################################
        ## START - WINDOW ATTRIBUTES
        ########################################################################

        ## REMOVE ==> STANDARD TITLE BAR
        UIFunctions.removeTitleBar(True)


        ## SET ==> WINDOW TITLE
        # self.setWindowTitle('Main Window - emachinery')
        UIFunctions.labelTitle(self, 'Main Window - emachinery - Idling')
        UIFunctions.labelDescription(self, 'GUI for electric machinery analysis')


        ## WINDOW SIZE ==> DEFAULT SIZE
        startSize = QSize(1440, 1080)
        self.resize(startSize)
        self.setMinimumSize(startSize)
        # UIFunctions.enableMaximumSize(self, 500, 720)


        ## ==> CREATE MENUS
        ########################################################################

        ## ==> TOGGLE MENU SIZE
        self.ui.btn_toggle_menu.clicked.connect(lambda: UIFunctions.toggleMenu(self, 220, True))


        ## ==> ADD CUSTOM MENUS
        self.ui.stackedWidget.setMinimumWidth(20)

        # files.qrc (url is defined in this qrc file), alternatively, you can specify the absolute locaiton.
        # UIFunctions.addNewMenu(self, "Name Plate Data",     "btn_namePlateData",     f"url({parent_dir}icons/16x16/cil-input.png)", True)
        UIFunctions.addNewMenu(self, "Name Plate Data",     "btn_namePlateData",     "url(:/16x16/icons/16x16/cil-input.png)", True)
        UIFunctions.addNewMenu(self, "Controller Tuning",   "btn_controllerTuning",  "url(:/16x16/icons/16x16/cil-loop.png)", True)
        UIFunctions.addNewMenu(self, "C-based Simulation",  "btn_cBasedSimulation",  "url(:/16x16/icons/16x16/cil-reload.png)", True)
        UIFunctions.addNewMenu(self, "Scope",               "btn_scope",             "url(:/16x16/icons/16x16/cil-chart-line.png)", True)
        UIFunctions.addNewMenu(self, "Plots",               "btn_plots",             "url(:/16x16/icons/16x16/cil-history.png)", True)
        UIFunctions.addNewMenu(self, "Parametric Analysis", "btn_parametricAnalysis","url(:/16x16/icons/16x16/cil-layers.png)", True)
        UIFunctions.addNewMenu(self, "Add User",            "btn_new_user",          "url(:/16x16/icons/16x16/cil-user-follow.png)", True)
        UIFunctions.addNewMenu(self, "HOME",                "btn_home",              "url(:/16x16/icons/16x16/cil-home.png)", True)
        UIFunctions.addNewMenu(self, "Py-based Simulation", "btn_pyBasedSimulation", "url(:/16x16/icons/16x16/cil-chevron-double-down.png)", False)
        UIFunctions.addNewMenu(self, "Custom Widgets",      "btn_widgets",           "url(:/16x16/icons/16x16/cil-equalizer.png)", False) # False goes to bottom

        # START MENU => SELECTION
        # UIFunctions.selectStandardMenu(self, "btn_home")
        UIFunctions.selectStandardMenu(self, "btn_cBasedSimulation")
        # if True:
        #     self.ui.stackedWidget.setCurrentWidget(self.ui.page_cBasedSimulation)
        #     UIFunctions.resetStyle(self, "btn_cBasedSimulation")
        #     UIFunctions.labelPage(self, "C-based Simulation")
        #     # btnWidget = self.ui.btn_cBasedSimulation
        #     # btnWidget.setStyleSheet(UIFunctions.selectMenu(btnWidget.styleSheet()))


        ## ==> START PAGE
        # self.ui.stackedWidget.setCurrentWidget(self.ui.page_home)
        self.ui.stackedWidget.setCurrentWidget(self.ui.page_cBasedSimulation)


        ## USER ICON ==> SHOW HIDE
        UIFunctions.userIcon(self, "CJH", "url(:/16x16/icons/16x16/cil-user.png)", True)
        # UIFunctions.userIcon(self, "CJH", "", True)



        ## ==> MOVE WINDOW / MAXIMIZE / RESTORE
        ########################################################################
        def moveWindow(event):
            # IF MAXIMIZED CHANGE TO NORMAL
            if UIFunctions.returStatus() == 1:
                UIFunctions.maximize_restore(self)

            # MOVE WINDOW
            if event.buttons() == Qt.LeftButton:
                self.move(self.pos() + event.globalPos() - self.dragPos)
                self.dragPos = event.globalPos()
                event.accept()

        # WIDGET TO MOVE
        self.ui.frame_label_top_btns.mouseMoveEvent = moveWindow


        ## ==> LOAD DEFINITIONS
        ########################################################################
        UIFunctions.uiDefinitions(self)


        ########################################################################
        ## END - WINDOW ATTRIBUTES
        ############################## ---/--/--- ##############################




        ########################################################################
        #                                                                      #
        ## START -------------- WIDGETS FUNCTIONS/PARAMETERS ---------------- ##
        #                                                                      #
        ## ==> USER CODES BELLOW                                              ##
        ########################################################################



        ## ==> QTableWidget RARAMETERS
        ########################################################################
        self.ui.tableWidget.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.Stretch)


        ## ==> AppFunctions Initialization
        AppFunctions.initialize_connections(self)
        # self.ui.ConsoleWidget.show() # TODO this used to give me a separate window console, but it


        # self.ui.ConsoleWidget.setStyleSheet(self.ui.ConsoleWidget.styleSheet() + "background: rgb(27, 29, 35);")
        # self.ui.plainTextEdit_ACMPlotLabels.setStyleSheet(self.ui.plainTextEdit_ACMPlotLabels.styleSheet() + "background: rgb(27, 29, 35);")
        # print('DEBUG:', self.ui.ConsoleWidget.styleSheet())
        # print('|||')
        # print('DEBUG:', self.ui.plainTextEdit_ACMPlotLabels.styleSheet())
        # self.ui.ConsoleWidget.setStyleSheet("background: rgb(27, 29, 35);")
        # self.ui.ConsoleWidget.setStyleSheet("background: transparent;")

        ########################################################################
        #                                                                      #
        ## END --------------- WIDGETS FUNCTIONS/PARAMETERS ----------------- ##
        #                                                                      #
        ############################## ---/--/--- ##############################


        ## SHOW ==> MAIN WINDOW
        ########################################################################
        # self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint) # always on top
        # self.setWindowState(self.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive) # https://stackoverflow.com/questions/42614623/how-to-make-a-pyside-window-pop-up-above-all-other-windows
        self.activateWindow()

        # self.setWindowState(QtCore.Qt.WindowMaximized)
        # self.show()
            # self.showFullScreen()
            # self.showMaximized()
            # self.ui.show()



        ########################################################################
        #                                                                      #
        ## START -------------- Add CSS for Pormoted Widgets ---------------- ##
        #                                                                      #
        ## ==> USER CODES BELLOW                                              ##
        ########################################################################

        # 颜色设置
        # 为什么初始化的时候设置css无效？而是必须在这里事后设置呢？？
        # 为什么无法修改ConsoleWidget和QCodeEditor的背景色呢？
        css = '''
        color: blue;
        background-color: yellow;
        selection-color: yellow;
        selection-background-color: blue;
        '''
        css = '''
        color: #ff6347;
        background-color: #272822;
        selection-color: #272822;
        selection-background-color: #d5d1c7;
        '''
        # print('{debug app_functions.py}: apply css loc 2')
        self.ui.ConsoleWidget.setStyleSheet(css)
        self.ui.plainTextEdit_ACMPlotLabels.setStyleSheet(css)
        self.ui.plainTextEdit_ACMPlotDetails.setStyleSheet(css)
        self.ui.plainTextEdit_LissajousPlotSignals.setStyleSheet(css)
        self.ui.plainTextEdit_PANames.setStyleSheet(css)
        self.ui.plainTextEdit_PAValues.setStyleSheet(css)

        self.ui.pushButton_ACMPlotHere
        self.ui.pushButton_ACMPlotHereAtTop
        self.ui.pushButton_runCBasedSimulation
        self.ui.pushButton_updateModel  
        self.ui.pushButton_pidTuner
        self.ui.pushButton_LissajousPlot
        self.ui.pushButton_getSignal
        self.ui.pushButton_runCBasedSimulation4PA
        self.ui.pushButton_plot4PA
        self.ui.pushButton_bodePlot

        ########################################################################
        #                                                                      #
        ## END --------------- Add CSS for Pormoted Widgets ----------------- ##
        #                                                                      #
        ############################## ---/--/--- ##############################

    ########################################################################
    ## MENUS ==> DYNAMIC MENUS FUNCTIONS
    ########################################################################
    def Button(self):
        # GET BT CLICKED
        btnWidget = self.sender()
        # print('DEBUG: ', btnWidget)

        dict_object = OD({
                'btn_home': 
                    {
                        'page': self.ui.page_home,
                        'label': "Home"
                    },
                'btn_new_user':
                    {
                        'page': self.ui.page_home,
                        'label': "New User"
                    },
                'btn_namePlateData':
                    {
                        'page': self.ui.page_namePlateData,
                        'label': "Name Plate Data",
                    },
                'btn_controllerTuning':
                    {
                        'page': self.ui.page_controllerTuning,
                        'label': "Controller Tuning",
                    },
                'btn_cBasedSimulation':
                    {
                        'page': self.ui.page_cBasedSimulation,
                        'label': "C-based Simulation",
                    },
                'btn_pyBasedSimulation':
                    {
                        'page': self.ui.page_PyBasedSimulation,
                        'label': "Py-based Simulation",
                    },
                'btn_plots':
                    {
                        'page': self.ui.page_plots,
                        'label': "Plots",
                    },
                'btn_scope':
                    {
                        'page': self.ui.page_scope,
                        'label': "Scope",
                    },
                'btn_parametricAnalysis':
                    {
                        'page': self.ui.page_parametricAnalysis,
                        'label': "Parametric Analysis",
                    },
                'btn_widgets':
                    {
                        'page': self.ui.page_widgets,
                        'label': "Custom Widgets"
                    }
            })

        for key, d in dict_object.items():

            if btnWidget.objectName() == key:
                self.ui.stackedWidget.setCurrentWidget(d['page'])
                UIFunctions.resetStyle(self, key)
                UIFunctions.labelPage(self, d['label'])
                btnWidget.setStyleSheet(UIFunctions.selectMenu(btnWidget.styleSheet(), this=self))

    ########################################################################
    ## START ==> APP EVENTS
    ########################################################################

    ## EVENT ==> MOUSE DOUBLE CLICK
    ########################################################################
    def eventFilter(self, watched, event):
        if watched == self.le and event.type() == QtCore.QEvent.MouseButtonDblClick:
            print("pos: ", event.pos())

    ## EVENT ==> MOUSE CLICK
    ########################################################################
    def mousePressEvent(self, event):
        self.dragPos = event.globalPos()
        if event.buttons() == Qt.LeftButton:
            print('Mouse click: LEFT CLICK')
        if event.buttons() == Qt.RightButton:
            print('Mouse click: RIGHT CLICK')
        if event.buttons() == Qt.MidButton:
            print('Mouse click: MIDDLE BUTTON')

    ## EVENT ==> KEY PRESSED
    ########################################################################
    def keyPressEvent(self, event):
        print('[emyMainWindow] Key: ' + str(event.key()) + ' | Text Press: ' + str(event.text()))

        if event.key() == QtCore.Qt.Key_Escape:
            self.close()
        elif event.key() == QtCore.Qt.Key_F11:
            if self.isMaximized():
                self.showNormal()
            else:
                self.showMaximized()
        elif event.key() == QtCore.Qt.Key_C:
            UIFunctions.selectStandardMenu(self, "btn_cBasedSimulation")
            UIFunctions.labelTitle(self, 'Main Window - emachinery - Jump to page_cBasedSimulation')
            self.ui.stackedWidget.setCurrentWidget(self.ui.page_cBasedSimulation)
        elif event.key() == QtCore.Qt.Key_S:
            UIFunctions.selectStandardMenu(self, "btn_scope")
            UIFunctions.labelTitle(self, 'Main Window - emachinery - Jump to page_scope')
            self.ui.stackedWidget.setCurrentWidget(self.ui.page_scope)
        elif event.key() == QtCore.Qt.Key_B:
            if self.ui.stackedWidget.currentIndex() == 4: # 4 -> page_cBasedSimulation
                UIFunctions.labelTitle(self, 'Main Window - emachinery - Build')
                AppFunctions.runCBasedSimulation(self)
        # elif event.key() == QtCore.Qt.Key_Z:
        #     AppFunctions.user_shortcut(self, event)
        else:
            super(emyMainWindow, self).keyPressEvent(event) # Do the default action on the parent class QLineEdit

    ## EVENT ==> Focus In
    ########################################################################
    def focusInEvent(self, event):
        # Do something with the event here
        super(emyMainWindow, self).focusInEvent(event) # Do the default action on the parent class QLineEdit

    ## EVENT ==> RESIZE EVENT
    ########################################################################
    def resizeEvent(self, event):
        self.resizeFunction()
        return super(emyMainWindow, self).resizeEvent(event)

    def resizeFunction(self):
        print('\t📗 [yellow]Height:[/]' + str(self.height()) + ' | [yellow]Width:[/] ' + str(self.width()))

    ########################################################################
    ## END ==> APP EVENTS
    ############################## ---/--/--- ##############################


# Watch Expressions
# ///////////////////////////////////////////////////////////////
# from PySide2.QtWidgets import QLineEdit
class expressionWindow(QWidget):
    def __init__(self):
        super().__init__()

        self.setWindowTitle('Figure: Watch Expressions')
        mainLayout = QVBoxLayout()
        self.input1 = QLineEdit()
        mainLayout.addWidget(self.input1)
        self.setLayout(mainLayout)


# IPython as Console
# ///////////////////////////////////////////////////////////////
class PyBlackBox_MainWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        # GET WIDGETS FROM "ui_main.py"
        # Load widgets inside MainWindow
        # ///////////////////////////////////////////////////////////////
        self.ui = Ui_PyBlackBox()
        self.ui.setupUi(self)

        # SET DEFAULT PAGE
        # ///////////////////////////////////////////////////////////////
        # self.ui.app_pages.setCurrentWidget(self.ui.home)
        self.ui.app_pages.setCurrentWidget(self.ui.page_console)

        # LOAD DICT SETTINGS FROM "settings.json" FILE
        # ///////////////////////////////////////////////////////////////
        self.settings = Settings()

        self.custom_btn_top = LeftMenuButton(
            self,
            "custom_btn_top",
            "images/icons_svg/icon_add_user.svg",
            "Add new friend"
        )
        self.custom_btn_top_2 = LeftMenuButton(
            parent=self,
            name="custom_btn_top_2_Scope",
            icon="images/icons_png/cil-chart-line.png",
            tooltip="View real-time waveforms"
        )
        self.custom_btn_top_3 = LeftMenuButton(
            parent=self,
            name="custom_btn_top_3_Console",
            icon="images/icons_svg/icon_send.svg",
            tooltip="IPython console"
        )
        self.custom_btn_bottom_1 = LeftMenuButton(
            self,
            "custom_btn_bottom_1",
            "images/icons_svg/icon_more_options.svg",
            "More options, test with many words"
        )
        self.custom_btn_bottom_2 = LeftMenuButton(
            self,
            "custom_btn_bottom_2",
            "images/icons_svg/icon_settings.svg",
            "Open settings"
        )
        self.ui.top_menus_layout.addWidget(self.custom_btn_top)
        self.ui.top_menus_layout.addWidget(self.custom_btn_top_2)
        self.ui.top_menus_layout.addWidget(self.custom_btn_top_3)
        self.ui.bottom_menus_layout.addWidget(self.custom_btn_bottom_1)
        self.ui.bottom_menus_layout.addWidget(self.custom_btn_bottom_2)

        # DEBUG
        self.custom_btn_top.clicked.connect(      lambda: print(f"{self.settings['app_name']}: clicked"))
        self.custom_btn_top.released.connect(     lambda: print(f"{self.custom_btn_top.objectName()}: released"))
        self.custom_btn_top_2.clicked.connect(    lambda: self.ui.app_pages.setCurrentWidget(self.ui.page_scope))
        self.custom_btn_top_3.clicked.connect(    lambda: self.ui.app_pages.setCurrentWidget(self.ui.page_console))
        self.custom_btn_bottom_1.clicked.connect( lambda: print(f"{self.custom_btn_bottom_1.objectName()}: clicked"))
        self.custom_btn_bottom_1.released.connect(lambda: print(f"{self.custom_btn_bottom_1.objectName()}: released"))
        self.custom_btn_bottom_2.clicked.connect( lambda: print(f"{self.custom_btn_bottom_2.objectName()}: clicked"))
        self.custom_btn_bottom_2.released.connect(lambda: print(f"{self.custom_btn_bottom_2.objectName()}: released"))

        # TOP USER BOX
        # Add widget to App
        # ///////////////////////////////////////////////////////////////
        self.top_user = TopUserInfo(self.ui.left_messages, 8, 64, "Hory", "Writing python codes")
        self.top_user.setParent(self.ui.top_user_frame)
        self.top_user.status.connect(self.status_change)

        # SET UI DEFINITIONS
        # Run set_ui_definitions() in the ui_functions.py
        # ///////////////////////////////////////////////////////////////
        ui_functions.UiFunctions.set_ui_definitions(self)

        # ADD MESSAGE BTNS / FRIEND MENUS
        # Add btns to page
        # ///////////////////////////////////////////////////////////////
        parent_dir = os.path.dirname(__file__)
        add_user = [ # CJH: the parent directory is emachinery, so I need to add guiv2/ here for user_image
            {
                "user_image" : f"{parent_dir}/images/users/cat.png",
                "user_name" : "Tom",
                "user_description" : "Did you see a mouse?",
                "user_status" : "online",
                "unread_messages" : 2,
                "is_active" : False
            },
            {
                "user_image" : f"{parent_dir}/images/users/mouse.png",
                "user_name" : "Jerry",
                "user_description" : "I think I saw a cat...",
                "user_status" : "busy",
                "unread_messages" : 1,
                "is_active" : False
            },
            {
                "user_image" : f"{parent_dir}/images/users/me.png",
                "user_name" : "Me From The Future",
                "user_description" : "Lottery result...",
                "user_status" : "invisible",
                "unread_messages" : 0,
                "is_active" : False
            }
        ]
        self.menu = FriendMessageButton
        def add_menus(self, parameters):
            id = 0
            for parameter in parameters:

                user_image = parameter['user_image']
                user_name = parameter['user_name']
                user_description = parameter['user_description']
                user_status = parameter['user_status']
                unread_messages = parameter['unread_messages']
                is_active = parameter['is_active']

                self.menu = FriendMessageButton(
                    id, user_image, user_name, user_description, user_status, unread_messages, is_active
                )
                self.menu.clicked.connect(self.btn_clicked)
                self.menu.released.connect(self.btn_released)
                self.ui.messages_layout.addWidget(self.menu)
                id += 1

        add_menus(self, add_user)


        # 颜色设置
        # 为什么初始化的时候设置css无效？而是必须在这里事后设置呢？？
        # 为什么无法修改ConsoleWidget和QCodeEditor的背景色呢？
        css = '''
        color: blue;
        background-color: yellow;
        selection-color: yellow;
        selection-background-color: blue;
        '''
        css = '''
        color: #ff6347;
        background-color: #272822;
        selection-color: #272822;
        selection-background-color: #d5d1c7;
        '''
        # print('{debug app_functions.py}: apply css loc 2')
        self.ui.PBB_ConsoleWidget.setStyleSheet(css)

        # SHOW MAIN WINDOW
        # ///////////////////////////////////////////////////////////////
        self.show()

    # SET USERNAME TO MAIN WINDOW
    # ///////////////////////////////////////////////////////////////
    def set_user_and_description(self, username):
        self.top_user.user_name = username
        print(f"User: {username} are logged!")

    # PRINT STATUS
    # ///////////////////////////////////////////////////////////////
    def status_change(self, status):
        print(f"send signal: {status}")

    # GET BTN CLICKED
    # ///////////////////////////////////////////////////////////////
    def btn_clicked(self):
        # GET BT CLICKED
        btn = self.sender()

        # UNSELECT CHATS
        ui_functions.UiFunctions.deselect_chat_message(self, btn.objectName())

        # SELECT CLICKED
        if btn.objectName():
            btn.reset_unread_message()
            ui_functions.UiFunctions.select_chat_message(self, btn.objectName())

        # LOAD CHAT PAGE
        if btn.objectName():
            # REMOVE CHAT
            for chat in reversed(range(self.ui.chat_layout.count())):
                self.ui.chat_layout.itemAt(chat).widget().deleteLater()
            self.chat = None

            # SET CHAT WIDGET
            self.chat = Chat(btn.user_image, btn.user_name, btn.user_description, btn.objectName(), self.top_user.user_name)

            # ADD WIDGET TO LAYOUT
            self.ui.chat_layout.addWidget(self.chat)

            # JUMP TO CHAT PAGE
            self.ui.app_pages.setCurrentWidget(self.ui.chat)

        # DEBUG
        print(f"Button {btn.objectName()}, clicked!")

    # GET BTN RELEASED
    # ///////////////////////////////////////////////////////////////
    def btn_released(self):
        # GET BT CLICKED
        btn = self.sender()
        print(F"Button {btn.objectName()}, released!")


    # RESIZE EVENT
    # Whenever the window is resized, this event will be triggered
    # ///////////////////////////////////////////////////////////////
    def resizeEvent(self, event):
        ui_functions.UiFunctions.resize_grips(self)

    # MOUSE CLICK EVENTS
    # ///////////////////////////////////////////////////////////////
    def mousePressEvent(self, event):
        # SET DRAG POS WINDOW
        self.dragPos = event.globalPos()


def startApp():

    app = QApplication(sys.argv)
    # window = LoginWindow()
    # window.show()

    # SHOW MAIN WINDOWB
    main = emyMainWindow()
    main.show()

    # SHOW Watch Expressions
    main.eW = None
    # main.eW = expressionWindow()
    # main.eW.show()

    # SHOW PBB as IPython Console
    main.pbb = PyBlackBox_MainWindow()
    main.pbb.top_user.label_user.setText('hory'.capitalize()) # 更新用户名
    main.pbb.show()

    if bool_use_PySide6:
        sys.exit(app.exec())
    elif bool_use_PyQt5:
        app.exec_()
    else:
        sys.exit(app.exec_())

if __name__ == "__main__":

    # TestStyle = '''
    # ConsoleWidget{
    #     background-color: rgb(100, 100, 100);
    # }
    # '''
    # app.setStyleSheet(TestStyle)

    # QtGui.QFontDatabase.addApplicationFont('fonts/segoeui.ttf')
    # QtGui.QFontDatabase.addApplicationFont('fonts/segoeuib.ttf')

    # print('Login window 2 elapsed:', time.time() - global_time)
    # window = LoginWindow()
    # window.show()

    print('\tElapsed: import modules:', time.time() - global_time)
    BOOL_MAIN_WINDOW_OPENED = True
    startApp() # QTimer.singleShot(1, startApp)



#coding:u8
from pylab import plt, mpl, np
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
# from pprint import pprint
from collections import OrderedDict as O
import pandas as pd
# plot style
#style = np.random.choice(plt.style.available); print(style); 
# plt.style.use('grayscale') # ['grayscale', u'dark_background', u'bmh', u'grayscale', u'ggplot', u'fivethirtyeight']
plt.style.use('ggplot')
# plot setting
mpl.rcParams['mathtext.fontset'] = 'stix'
mpl.rcParams['font.family'] = 'STIXGeneral'
mpl.rcParams['legend.fontsize'] = 12.5
# mpl.rcParams['legend.family'] = 'Times New Roman'
mpl.rcParams['font.family'] = ['Times New Roman']
mpl.rcParams['font.size'] = 14.0
# mpl.style.use('classic')
font = {'family' : 'Times New Roman', #'serif',
        'color' : 'darkblue',
        'weight' : 'normal',
        'size' : 14,}
textfont = {'family' : 'Times New Roman', #'serif',
            'color' : 'darkblue',
            'weight' : 'normal',
            'size' : 11.5,}

import os
from time import sleep

######################
# Plotting
def get_axis(cNr):
    # fig, axes = plt.subplots(ncols=cNr[0], nrows=cNr[1], dpi=150, sharex=True);
    fig, axes = plt.subplots(ncols=cNr[0], nrows=cNr[1], sharex=True, figsize=(16*0.8, 9*0.8), dpi=80, facecolor='w', edgecolor='k');
    fig.subplots_adjust(right=0.95, bottom=0.1, top=0.95, hspace=0.2, wspace=0.02)    
    # fig.subplots_adjust(right=0.85, bottom=0.1, top=0.95, hspace=0.25)
    if sum(cNr)<=2:
        return axes
    else:
        return axes.ravel()

def plot_key(ax, key, df):
    ax.plot(time, df[key].values, '-', lw=1)
    ax.set_ylabel(key, fontdict=font)

def plot_it(ax, ylabel, d, time=None):
    count = 0
    for k, v in d.items():
        if count == 0:
            count += 1
            # ax.plot(time, v, '--', lw=2, label=k)
            ax.plot(time, v, '-.', lw=1)
        else:
            # ax.plot(time, v, '-', lw=2, label=k)
            ax.plot(time, v, '-.', lw=1)

    # ax.legend(loc='lower right', shadow=True)
    # ax.legend(bbox_to_anchor=(1.08,0.5), borderaxespad=0., loc='center', shadow=True)
    ax.set_ylabel(ylabel, fontdict=font)
    # ax.set_xlim(0,35) # shared x
    # ax.set_ylim(0.85,1.45)

def get_data_frame(path='.'):

    # info.dat
    df_info = pd.read_csv(path+"/dat/info.dat", na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
    data_file_name = df_info['DATA_FILE_NAME'].values[0].strip()

    # 这个大的数据文件产生有滞后，可能文件还没生成，程序就已经跑到这里开始调用read_csv了！
    # 所以要判断一下，数据文件应该是在info.dat之后生成的。
    # while True:
    #     print('st_mtime 1:', os.stat(path+'/dat/'+data_file_name).st_mtime )
    #     print('st_mtime 2:', os.stat(path+"/dat/info.dat").st_mtime)
    #     if os.stat(path+'/dat/'+data_file_name).st_mtime < os.stat(path+"/dat/info.dat").st_mtime:
    #         print('Sleep in ACMPlot.py')
    #         sleep(0.1)
    #         break    

    # ???.dat
    df_profiles = pd.read_csv(path+'/dat/'+data_file_name, na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
    no_samples = df_profiles.shape[0]
    no_traces  = df_profiles.shape[1]
    # print(data_file_name)

    print('read in', path+'/dat/'+data_file_name)
    print(df_profiles.shape)
    # print(df_profiles)

    return df_info, df_profiles, no_samples, no_traces 

if __name__ == '__main__':

    df_info, df_profiles, no_samples, no_traces = get_data_frame()
    print(df_info, 'Simulated time: %g s.'%(no_samples * df_info['CL_TS'].values[0] * df_info['DOWN_SAMPLE'].values[0]), 'Key list:', sep='\n')
    for key in df_profiles.keys():
        print('\t', key)

    time = np.arange(1, no_samples+1) * df_info['DOWN_SAMPLE'].values[0] * df_info['CL_TS'].values[0]

    ax_list = []
    for i in range(0, no_traces, 6):
        ax_list += list(get_axis((1,6)))

    for idx, key in enumerate(df_profiles.keys()):
        plot_it(ax_list[idx], key, O([
                                        (str(idx), df_profiles[key]),  
                                        # (str(idx), df_profiles[key]),  
                                        ]), time)
        if (idx+1)%6 == 0:
            ax_list[idx].set_xlabel('time [s]', fontdict=font)


    plt.show()
    quit()





from dataclasses import dataclass
from pylab import plt, mpl, np
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes, mark_inset
from collections import OrderedDict as OD
import pandas as pd
import os
mpl.rcParams['legend.fontsize'] = 10
mpl.rcParams['font.family'] = ['serif'] # default is sans-serif
mpl.rcParams['font.serif'] = ['Times New Roman']
font = {'family':'Times New Roman', 'weight':'normal', 'size':10}

# plt.style.use('ggplot') 
# plt.style.use('grayscale') # print plt.style.available # get [u'dark_background', u'bmh', u'grayscale', u'ggplot', u'fivethirtyeight']
mpl.style.use('classic')


######################
# Data
def load_dataframe(path='./', index=0):

    df_list = []
    fname_list = []
    for fname in os.listdir(path):
        if '.dat' in fname and 'info' not in fname and 'pdf' not in fname:
            print('--------------', fname)
            fname_list.append(fname)
            df = pd.read_csv(path+fname, na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
            df_list.append(df)

    # index = 0
    df    = df_list[index]
    fname = fname_list[index]
    no_samples = df.shape[0]
    no_traces  = df.shape[1]
    df_info = pd.read_csv(path+"info.dat", na_values = ['1.#QNAN', '-1#INF00', '-1#IND00'])
    time = np.arange(1,no_samples+1,1) * float(df_info['DOWN_SAMPLE']) * float(df_info['CL_TS'])  # sampling frequency is 10e3
    print('---- List of Keys:')
    for key in df.keys():
        print('\t', key)

    return df_list, time, fname

######################
# Plotting
def locallyZoomIn(ax, data_zoomed, ylim=None, loc1=None, loc2=None, y_axis_shrink=None, **kwarg):
    # extent = [-3, 4, -4, 3]

    if(y_axis_shrink!=None):
        temp_tuple = (ylim[0]*y_axis_shrink, ylim[1]*y_axis_shrink)
        ylim = temp_tuple
        for ind, el in enumerate(data_zoomed):
            temp_tuple = (el[0], el[1]*y_axis_shrink)
            data_zoomed[ind] = temp_tuple

    axins = zoomed_inset_axes(ax, **kwarg) # zoom = 
    # axins.plot(Z2, extent=extent, interpolation="nearest", origin="lower")
    for ind, el in enumerate(data_zoomed):
        if ind==1:
            if len(data_zoomed)>1:
                axins.plot(el[0],el[1], 'r-', alpha=0.7, linewidth=0.8) # '--'
            else:
                axins.plot(el[0],el[1], 'k-', alpha=0.7, lw=0.8)
        else:
            # axins.plot(*el, lw=2)
            axins.plot(el[0],el[1], lw=1)

    # sub region of the original image
    # x1, x2, y1, y2 = 1.2*pi, 1.25*pi, -0.78, -0.54
    # axins.set_xlim(x1, x2)
    # axins.set_ylim(y1, y2)

    if ylim!=None:
        axins.set_ylim(ylim)

    plt.xticks(np.arange(  axins.viewLim.intervalx[0],
                        axins.viewLim.intervalx[1],
                        axins.viewLim.width/2), visible=False)
    plt.yticks(np.arange(  axins.viewLim.intervaly[0],
                        axins.viewLim.intervaly[1],
                        axins.viewLim.height/4), visible=False)

    if(y_axis_shrink==None):
        # draw a bbox of the region of the inset axes in the parent axes and
        # connecting lines between the bbox and the inset axes area
        if loc1!=None and loc2!=None:
            mark_inset(ax, axins, loc1=loc1, loc2=loc2, fc="none", ec="0.5")
        else:
            mark_inset(ax, axins, loc1=2, loc2=3, fc="none", ec="0.5")
        plt.draw()
    return axins
def plot_it(ax, time, ylabel, OrDi, bool_legend=False):
    cnt_temp = 0
    for k, v in OrDi.items():
        cnt_temp += 1
        if cnt_temp == 2:
            ax.plot(time, v, 'r-', alpha=0.7, lw=0.8)
        elif cnt_temp == 3:
            print('A thrid trace is plotted in blue')
            ax.plot(time, v, 'b-', alpha=0.7, lw=0.8)
        else:
            ax.plot(time, v, 'k-', alpha=0.7, lw=0.8) # label=k
    if bool_legend:
        # ax.legend(loc='lower right', shadow=True)
        ax.legend(bbox_to_anchor=(1.08,0.5), borderaxespad=0., loc='center', shadow=True)
    ax.set_ylabel(ylabel, fontdict=font)
    # ax.set_xlim(0,35) # shared x
    # ax.set_ylim(0.85,1.45)

@dataclass
class CJHStylePlot():
    path2info_dot_dat: str
    nrows: int
    ncols: int          = 1
    sharex: bool        = True
    figsize: tuple      = (8, 9)
    facecolor: str      = 'w'
    edgecolor: str      = 'k'

    def __post_init__(self):
        # this is a directory   # this is path to a file
        self.dir2info_dot_dat = os.path.dirname(os.path.abspath(self.path2info_dot_dat)) + '/'
        print('path2info_dot_dat:', self.path2info_dot_dat)
        print('dir2info_dot_dat:', self.dir2info_dot_dat)

    def load(self):
        self.df_list, self.time, self.fname = load_dataframe(self.dir2info_dot_dat)
        return self.df_list, self.time, self.fname

    def plot(self, list_of_ylabel, list_of_signalNames, customize_plot=lambda axes_v:None, index=0):
        if self.df_list is None:
            self.load()

        df, time, fname = self.df_list[index], self.time, self.fname
        fig, axes_v = plt.subplots(  ncols=     self.ncols,
                                     nrows=     self.nrows,
                                     sharex=    self.sharex,
                                     figsize=   self.figsize,
                                     facecolor= self.facecolor,
                                     edgecolor= self.edgecolor,
                                   ); # modifying dpi would change the size of fonts
        # fig.subplots_adjust(wspace=0.01)

        for i, ylabel, signalNames in zip(range(self.nrows), list_of_ylabel, list_of_signalNames):
            ax = axes_v[i]
            OrDi = OD()
            for ind, signalName in enumerate(signalNames):
                OrDi[str(ind)] = df[signalName]
            plot_it(ax, time, ylabel, OrDi)

        for ax in axes_v:
            ax.grid(True)
            # ax.axvspan(14, 29, facecolor='r', alpha=0.1)
            ax.set_yticklabels([f'{el:g}' for el in ax.get_yticks()], font) # 为了修改ytick的字体  # https://matplotlib.org/3.3.3/api/_as_gen/matplotlib.axes.Axes.set_yticklabels.html

        # axes_v[-1].set_xlim((0,130))
        # axes_v[-1].set_xticks(np.arange(0, 201, 20))
        axes_v[-1].set_xlabel('Time, $t$ [s]', fontdict=font)
        axes_v[-1].set_xticklabels(axes_v[-1].get_xticks(), font)

        customize_plot(axes_v)

        # print(self.path2info_dot_dat)
        # print(fname[:-4])
        # print(fname)
        # fig.tight_layout() # tight layout is bad if you have too many subplots. Use bbox_inches instead
        path2outputFile = self.dir2info_dot_dat+fname[:-4] + f'-{index+1:03d}'
        fig.savefig(              f'{path2outputFile}.pdf', dpi=300, bbox_inches='tight', pad_inches=0)
        os.system('sumatrapdf ' + f'{path2outputFile}.pdf')
        os.system(f'pdfcrop         {path2outputFile}.pdf \
                                    {path2outputFile}-crop.pdf')
        # plt.show()

if __name__ == '__main__':
    # import emachinery
    # CJHStylePlot = emachinery.acmsimcv5.CJHAcademicPlot.__cjhAcademicPlotSettings.CJHStylePlot
    csp = CJHStylePlot( path2info_dot_dat=r'D:\DrH\Codes\emachineryTestPYPI\emachinery\acmsimcv5\dat\IM_Marino05_PI-Ohtani/info.dat', 
                        nrows=6)

    list_of_ylabel = [
      'Speed\n[rpm]',
      '$\\alpha$-axis Flux\n[Wb]',
      'Torque Current\n[A]',
      'Offset Voltage\n[V]',
      'Magn. Current\n[A]',
      'Load Torque\n[Nm]',
    ]

    list_of_signalNames = [
    ######################################
    ("ACM.rpm_cmd",
    "ACM.rpm",
    "marino.xOmg*ELEC_RAD_PER_SEC_2_RPM",
    ),
    ######################################
    ("ACM.psi_Amu",
    "ohtani.psi_2_real_output[0]",
    ),
    ######################################
    ("CTRL.I->iDQ_cmd[1]",
    # "ACM.iTs",
    "CTRL.I->iDQ[1]",
    ) ,
    ######################################
    ("ohtani.correction_integral_term[0]",
    "ohtani.correction_integral_term[1]",
    ) ,
    ######################################
    ("CTRL.I->iDQ_cmd[0]",
    # "ACM.iMs",
    "CTRL.I->iDQ[0]",
    ) ,
    ######################################
    ("ACM.TLoad",
    "marino.xTL",
    ),
    ]

    def customize_plot(axes_v):
        axes_v[0].set_ylim([-300, 300])
    csp.plot(list_of_ylabel, list_of_signalNames, customize_plot=customize_plot)




# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

import json
import os

# APP SETTINGS
class Settings(object):
    # APP PATH
    # ///////////////////////////////////////////////////////////////
    json_file = "settings.json"
    # app_path = os.path.abspath(os.getcwd())
    # settings_path = os.path.normpath(os.path.join(app_path, json_file))
    settings_path = os.path.dirname(__file__) + './' + json_file
    if not os.path.isfile(settings_path):
        print(f"WARNING: \"settings.json\" not found! check in the folder {settings_path}")

    def __init__(self):
        super(Settings, self).__init__()

        # DICTIONARY WITH SETTINGS
        # Just to have objects references
        self.items = {}

        # DESERIALIZE
        self.deserialize()

    # SERIALIZE JSON
    # ///////////////////////////////////////////////////////////////
    def serialize(self):
        # WRITE JSON FILE
        with open(self.settings_path, "w", encoding='utf-8') as write:
            json.dump(self.items, write, indent=4)

    # DESERIALIZE JSON
    # ///////////////////////////////////////////////////////////////
    def deserialize(self):
        # print(self.settings_path)
        # READ JSON FILE
        with open(self.settings_path, "r", encoding='utf-8') as reader:
            settings = json.loads(reader.read())
            self.items = settings


from shutil import copyfile
def main():
    # Do not copy ACMSim.h
    for fname in ['pid_regulator.c',
                  'pid_regulator.h',
                  'shared_flux_estimator.c',
                  'shared_flux_estimator.h',
                  'pmsm_controller.c',
                  'pmsm_controller.h',
                  'pmsm_observer.c',
                  'pmsm_observer.h',
                  'pmsm_comm.c',
                  'pmsm_comm.h',
                  'global_variables_definitions.c',
                  'utility.c',
                  'ACMConfig.h',
                  'sweep_frequency.h',
               ]:
        copyfile(f'c/{fname}', rf'D:\DrH\CCSWS10\pangu-debug\The acmsimcv5/{fname}')
if __name__ == '__main__':
    main()



#!/usr/bin/python3
# QcodeEditor.py by acbetter.
# -*- coding: utf-8 -*-
# from main import *
from app_modules import *

if bool_use_PySide6 == True:

    from PySide6.QtGui import QColor, QPainter, QTextFormat

elif bool_use_PyQt5:
    #     from PyQt5 import QtCore, QtGui
    #     from PyQt5.QtCore import Qt, QRect, QSize
    #     from PyQt5.QtWidgets import QWidget, QPlainTextEdit, QTextEdit
    from PyQt5.QtGui import QColor, QPainter, QTextFormat
    #     from PyQt5.QtWidgets import QStyleOption, QStyle
else:
    #     from PySide2 import QtCore, QtGui
    #     from PySide2.QtCore import Qt, QRect, QSize
    #     from PySide2.QtWidgets import QWidget, QPlainTextEdit, QTextEdit
    from PySide2.QtGui import QColor, QPainter, QTextFormat
    #     from PySide2.QtWidgets import QStyleOption, QStyle

class QLineNumberArea(QWidget):
    def __init__(self, editor):
        super().__init__(editor)
        self.codeEditor = editor

    def sizeHint(self):
        return QSize(self.editor.lineNumberAreaWidth(), 0)

    def paintEvent(self, event):
        self.codeEditor.lineNumberAreaPaintEvent(event)


class QCodeEditor(QPlainTextEdit):
    def __init__(self, parent=None):
        super(QCodeEditor, self).__init__(parent)
        self.lineNumberArea = QLineNumberArea(self)
        self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
        self.updateRequest.connect(self.updateLineNumberArea)
        self.cursorPositionChanged.connect(self.highlightCurrentLine)
        self.updateLineNumberAreaWidth(0)

        # print(self.style())
        # stylesheet
        # self.setBackground(QColor(Qt.yellow).lighter(160))

        # useless for guiv2?
#         css = '''
# color: blue;
# background-color: yellow;
# selection-color: yellow;
# selection-background-color: blue;
# '''
#         self.setStyleSheet(css)

        # self.setStyleSheet("background: rgb(27, 29, 35);")
        # print('DEBUG: QcodeEditor:', self.styleSheet())

        # useless to set background color for promoted widget
        # self.setAttribute(QtCore.Qt.WA_StyledBackground, True) # https://stackoverflow.com/questions/54965088/pyqt-promoted-widget-background-issues?noredirect=1&lq=1

    # def paintEvent(self, evt):
    #     # no effect???
    #     # Stefan Reinhardt from https://stackoverflow.com/questions/7276330/qt-stylesheet-for-custom-widget
    #     super(QCodeEditor, self).paintEvent(evt)
    #     opt = QStyleOption()
    #     opt.initFrom(self)
    #     p = QtGui.QPainter(self)
    #     s = self.style()
    #     s.drawPrimitive(QStyle.PE_Widget, opt, p, self) 

    def lineNumberAreaWidth(self):
        digits = 1
        max_value = max(1, self.blockCount())
        while max_value >= 10:
            max_value /= 10
            digits += 1
        space = 3 + self.fontMetrics().width('9') * digits
        return space

    def updateLineNumberAreaWidth(self, _):
        self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

    def updateLineNumberArea(self, rect, dy):
        if dy:
            self.lineNumberArea.scroll(0, dy)
        else:
            self.lineNumberArea.update(0, rect.y(), self.lineNumberArea.width(), rect.height())
        if rect.contains(self.viewport().rect()):
            self.updateLineNumberAreaWidth(0)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        cr = self.contentsRect()
        self.lineNumberArea.setGeometry(QRect(cr.left(), cr.top(), self.lineNumberAreaWidth(), cr.height()))

    def highlightCurrentLine(self):
        extraSelections = []
        if not self.isReadOnly():
            selection = QTextEdit.ExtraSelection()
            # lineColor = QColor(Qt.yellow).lighter(160) 
            # lineColor = QColor(Qt.gray).lighter(160) # TODO: change to different color for dark stylesheet
            # lineColor = QColor('#659B91')
            lineColor = QColor('#615F4E')
            selection.format.setBackground(lineColor)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extraSelections.append(selection)
        self.setExtraSelections(extraSelections)

    def lineNumberAreaPaintEvent(self, event):
        painter = QPainter(self.lineNumberArea)

        painter.fillRect(event.rect(), Qt.lightGray)

        block = self.firstVisibleBlock()
        blockNumber = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()

        # Just to make sure I use the right font
        height = self.fontMetrics().height()
        while block.isValid() and (top <= event.rect().bottom()):
            if block.isVisible() and (bottom >= event.rect().top()):
                number = str(blockNumber + 1)
                painter.setPen(Qt.black)
                painter.drawText(0, top, self.lineNumberArea.width(), height, Qt.AlignRight, number)

            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            blockNumber += 1


if __name__ == '__main__':
    import sys
    from PyQt5.QtWidgets import QApplication

    app = QApplication(sys.argv)
    codeEditor = QCodeEditor()
    codeEditor.setStyleSheet("background: rgb(27, 29, 35);")
    # codeEditor.setStyleSheet("background: transparent;")
    codeEditor.show()
    sys.exit(app.exec_())




from app_modules import *

# from main import *
# if bool_use_PyQt5:
#     from PyQt5 import QtGui, QtCore
#     from PyQt5.QtWidgets import QApplication, QWidget, QStyleOption, QStyle
# else:
#     from PySide2 import QtGui, QtCore
#     from PySide2.QtWidgets import QApplication, QWidget, QStyleOption, QStyle


# https://stackoverflow.com/questions/11513132/embedding-ipython-qt-console-in-a-pyqt-application
# from qtconsole.qt import QtGui # This works with qtconsole-4.6.0
from qtconsole.rich_jupyter_widget import RichJupyterWidget
from qtconsole.inprocess import QtInProcessKernelManager

# import logging

class ConsoleWidget(RichJupyterWidget):

    # def __init__(self, customBanner=None, *args, **kwargs):
    def __init__(self, parent=None, *args, **kwargs):

        # print('DeBUG consolewidget:', parent)

        # QWidget.__init__(self, parent)
        r''' I have to comment this out because of following error:
            D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2>python main.py
            Traceback (most recent call last):
              File "main.py", line 289, in <module>
                window = MainWindow() # ui=QUiLoader().load("GUI_BASE.ui", None)
              File "main.py", line 25, in __init__
                self.ui.setupUi(self)
              File "D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2\ui_main.py", line 1294, in setupUi
                self.ConsoleWidget = ConsoleWidget(self.page_namePlateData)
              File "D:\DrH\Codes\emachineryTestPYPI\emachinery\guiv2\consolewidget.py", line 16, in __init__
                QWidget.__init__(self, parent)
            RuntimeError: You can't initialize an object twice!
            QBasicTimer::start: QBasicTimer can only be used with threads started with QThread
        '''

        super(ConsoleWidget, self).__init__(*args, **kwargs)

        # useless for guiv2?
        # self.setStyleSheet("background: rgb(27, 29, 35);")

            #         css = '''
            # color: blue;
            # background-color: yellow;
            # selection-color: yellow;
            # selection-background-color: blue;
            # '''
            #         self.setStyleSheet(css)

        # if customBanner is not None:
        #     self.banner = customBanner
        self.banner = "This is an embedded qtconsole.\nSuggested commands:\n\tvars(emy)\n\n"
                    # [attr for attr in dir(emy) if not callable(getattr(emy, attr)) and not attr.startswith('__')]
                    # https://stackoverflow.com/questions/1398022/looping-over-all-member-variables-of-a-class-in-python

        # 无法修改字体大小
        self.font_size = 6+3
        # self.font_sizeInt = 9+3
        # print(self.font)
        self.font.setPointSize(24) # https://doc.qt.io/qt-5/qfont.html#setPointSize

        self.kernel_manager = kernel_manager = QtInProcessKernelManager()
        kernel_manager.start_kernel(show_banner=False)
        # kernel_manager.kernel.log.setLevel(logging.CRITICAL) #
        kernel_manager.kernel.gui = 'qt'
        self.kernel_client = kernel_client = self._kernel_manager.client()
        kernel_client.start_channels()

        def stop():
            kernel_client.stop_channels()
            kernel_manager.shutdown_kernel()
            guisupport.get_app_qt().exit()
        self.exit_requested.connect(stop)

        # # useless to set background color for promoted widget
        # self.setAttribute(QtCore.Qt.WA_StyledBackground, True) # https://stackoverflow.com/questions/54965088/pyqt-promoted-widget-background-issues?noredirect=1&lq=1

    # def paintEvent(self, evt):
    #     # no effect???
    #     # Stefan Reinhardt from https://stackoverflow.com/questions/7276330/qt-stylesheet-for-custom-widget
    #     super(ConsoleWidget, self).paintEvent(evt)
    #     opt = QStyleOption()
    #     opt.initFrom(self)
    #     opt.init(self)
    #     p = QtGui.QPainter(self)
    #     s = self.style()
    #     s.drawPrimitive(QStyle.PE_Widget, opt, p, self) 

    def push_vars(self, variableDict):
        """
        Given a dictionary containing name / value pairs, push those variables
        to the Jupyter console widget
        """
        self.kernel_manager.kernel.shell.push(variableDict)

    def clear(self):
        """
        Clears the terminal
        """
        self._control.clear()

        # self.kernel_manager

    def print_text(self, text):
        """
        Prints some plain text to the console
        """
        self._append_plain_text(text)

    def execute_command(self, command):
        """
        Execute a command in the frame of the console widget
        """
        self._execute(command, False)


if __name__ == '__main__':

    app = QApplication([])
    widget = ConsoleWidget()
    # widget.setStyleSheet("background: rgb(27, 29, 35);")
    widget.show()
    if bool_use_PyQt5:
        app.exec_()
    else:
        sys.exit(app.exec_())



# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# DEFAULT PACKAGES
# ///////////////////////////////////////////////////////////////
# import os
import random

# IMPORT / GUI, SETTINGS AND WIDGETS
# ///////////////////////////////////////////////////////////////
# Packages
# from app.packages.pyside_or_pyqt import * # Qt
# from app.packages.widgets import * # Widgets
# from PySide2.QtWidgets import *
from app_modules import * # Settings
# GUI
# from ui_page_messages import Ui_chat_page # MainWindow
# from message import Message # MainWindow


# MAIN WINDOW
# ///////////////////////////////////////////////////////////////
class Chat(QWidget):
    def __init__(
        self,
        user_image,
        user_name,
        user_description,
        user_id,
        my_name,
    ):
        QWidget.__init__(self)

        self.page = Ui_chat_page()
        self.page.setupUi(self)

        # UPDATE INFO
        self.page.user_image.setStyleSheet("#user_image { background-image: url(\"" + os.path.normpath(user_image).replace("\\", "/") + "\") }")
        self.page.user_name.setText(user_name)
        self.page.user_description.setText(user_description)

        # CHANGE PLACEHOLDER TEXT
        format_user_name = user_name.replace(" ", "_").replace("-", "_")
        format_user_name = format_user_name.lower()
        self.page.line_edit_message.setPlaceholderText(f"Message #{str(format_user_name).lower()}")

        # ENTER / RETURN PRESSED
        self.page.line_edit_message.keyReleaseEvent = self.enter_return_release

        # ENTER / RETURN PRESSED
        self.page.btn_send_message.clicked.connect(self.send_message)

        # MESSAGES
        self.messages = [
            f"Hi {my_name.capitalize()}, how are you?",
            f"Hello {my_name.capitalize()}, how are you today?",
            f"{my_name.capitalize()}, do you know if it is going to rain today?",
            f"{my_name.capitalize()}, how is your day?",
            f"{my_name.capitalize()}, do you remember that you owe me $100? Humm..."
        ]

        # SEND USER MESSAGE
        self.send_by_friend()

    # ENTER / RETURN SEND MESSAGE
    def enter_return_release(self, event):
        if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
            self.send_message()

    # SEND MESSAGE
    def send_message(self):
        if self.page.line_edit_message.text() != "":
            self.message = Message(self.page.line_edit_message.text(), True)
            self.page.chat_messages_layout.addWidget(self.message, Qt.AlignCenter, Qt.AlignBottom)
            self.page.line_edit_message.setText("")

            # SCROLL TO END            
            QTimer.singleShot(10, lambda: self.page.messages_frame.setFixedHeight(self.page.chat_messages_layout.sizeHint().height()))
            QTimer.singleShot(15, lambda: self.scroll_to_end())

            # SEND USER MESSAGE
            QTimer.singleShot(1000, lambda: self.send_by_friend())

    # SEND MESSAGE BY FRIEND
    def send_by_friend(self):
        self.message = Message(random.choice(self.messages), False)
        self.page.chat_messages_layout.addWidget(self.message, Qt.AlignCenter, Qt.AlignBottom)
        self.page.line_edit_message.setText("")

        # SCROLL TO END            
        QTimer.singleShot(10, lambda: self.page.messages_frame.setFixedHeight(self.page.chat_messages_layout.sizeHint().height()))
        QTimer.singleShot(15, lambda: self.scroll_to_end())


    def scroll_to_end(self):
        # SCROLL TO END
        self.scroll_bar = self.page.chat_messages.verticalScrollBar()
        self.scroll_bar.setValue(self.scroll_bar.maximum())


# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT
# ///////////////////////////////////////////////////////////////
# Packages
# from PySide2.QtWidgets import *
# from PySide2.QtCore import Signal
# from PySide2.QtGui import QPainterPath, QPen, QImage
from app_modules import * # Settings
# Modules
import os

# TOP USER BOX
# Top box with name, description and status
# ///////////////////////////////////////////////////////////////
class TopUserInfo(QWidget):
    status = Signal(str)
    def __init__(self, parent, left, top, my_name, my_description):
        QWidget.__init__(self)

        # ICON PATH
        # ///////////////////////////////////////////////////////////////
        image = "images/users/me.png"
        icon_settings = "images/icons_svg/icon_settings.svg"
        app_path = os.path.abspath(os.getcwd())
        image_path = os.path.join(app_path, image)
        icon_settings_path = os.path.join(app_path, icon_settings)

        # INITIAL SETUP
        # ///////////////////////////////////////////////////////////////
        self.setGeometry(0, 0, 240, 60)
        self.setObjectName("top_text_box")
        self.setStyleSheet("#top_text_box { background-color: #F00000 }")

        # CUSTOM PARAMETERS
        # ///////////////////////////////////////////////////////////////
        self.user_name = my_name
        self.user_description = my_description
        self.user_status = "online"
        self.user_image = image_path
        self.icon_settings = icon_settings
        self._status_color = "#46b946"

        # DRAW BASE FRAME
        # ///////////////////////////////////////////////////////////////
        self.setup_ui()

        # IMAGE FRAME EVENTS
        # ///////////////////////////////////////////////////////////////
        self.user_overlay.mousePressEvent = self.mouse_press
        self.user_overlay.enterEvent = self.mouse_enter
        self.user_overlay.leaveEvent = self.mouse_leave

        # SETUP STATUS BOX
        # ///////////////////////////////////////////////////////////////
        self.status_box = _ChangeStatus(parent)
        self.status_box.move(left, top)
        self.status_box.focusOutEvent = self.lost_focus_status_box
        self.status_box.line_edit.focusOutEvent = self.lost_focus_line_edit
        self.status_box.line_edit.keyReleaseEvent = self.change_description
        self.status_box.hide()
        self.status_box.status.connect(self.change_status)       
        
    # CHANGE USER STATUS
    # Change when is connected with status signal
    # ///////////////////////////////////////////////////////////////
    def change_status(self, status):
        # CHANGE STATUS
        if status == "online":
            self._status_color = "#46b946"
            self.repaint()
        elif status == "idle":
            self._status_color = "#ff9955"
            self.repaint()
        elif status == "busy":
            self._status_color = "#a02c2c"
            self.repaint()
        elif status == "invisible":
            self._status_color = "#808080"
            self.repaint()
        # EMIT SIGNAL
        self.status.emit(status)

    # CHANGE TEXT DESCRIPTION
    # ///////////////////////////////////////////////////////////////
    def change_description(self, event):        
        if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter:
            self.label_description.setText(self.status_box.line_edit.text())
            self.status_box.line_edit.setText("")
            self.status_box.hide()
        
    # SHO HIDE DIALOP POPUP
    # ///////////////////////////////////////////////////////////////
    # HIDE LINE EDIT WHEN LOST FOCUS
    def lost_focus_status_box(self, event):
        if not self.status_box.line_edit.hasFocus():
            self.status_box.hide()
            self.status_box.line_edit.setText("")

    # HIDE WHEN LOST FOCUS
    def lost_focus_line_edit(self, event):
        if not self.status_box.hasFocus():
            self.status_box.hide()
            self.status_box.line_edit.setText("")

    # OPEN STATUS BOX POPUP
    # ///////////////////////////////////////////////////////////////
    def mouse_press(self, event):
        if self.status_box.isVisible():
            self.status_box.hide()
            self.status_box.line_edit.setText("")
        else:
            self.status_box.show()
            self.status_box.line_edit.setFocus()

    # SHOW ICON
    # ///////////////////////////////////////////////////////////////
    def mouse_enter(self, event):
        self.user_overlay.setPixmap(self.icon_settings)

    # HIDE ICON
    # ///////////////////////////////////////////////////////////////
    def mouse_leave(self, event):
        self.user_overlay.setPixmap(None)

    # SETUP WIDGETS
    # ///////////////////////////////////////////////////////////////
    def setup_ui(self):
        # LAYOUT AND BORDER
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(10,10,10,0)
        self.border = QFrame(self)
        self.layout.addWidget(self.border)
        
        # FRAME IMAGE
        self.user_overlay = QLabel(self.border)
        self.user_overlay.setGeometry(0, 5, 40, 40)
        self.user_overlay.setCursor(QCursor(Qt.PointingHandCursor))
        self.user_overlay.setAlignment(Qt.AlignCenter)
        opacity = QGraphicsOpacityEffect(self)
        opacity.setOpacity(0.75)
        self.user_overlay.setGraphicsEffect(opacity)

        # FRAME TEXT
        self.text_frame = QFrame(self.border)
        self.text_frame.setGeometry(50, 0, 170, 50)

        # USER NAME
        self.label_user = QLabel(self.text_frame)
        self.label_user.setGeometry(0, 8, self.text_frame.width(), 20)
        self.label_user.setAlignment(Qt.AlignVCenter)        
        self.label_user.setText(self.user_name.capitalize())
        self.label_user.setStyleSheet("color: #bdff00; font: 700 10pt 'Segoe UI';")

        # USER STATUS
        self.label_description = QLabel(self.text_frame)
        self.label_description.setGeometry(0, 22, self.text_frame.width(), 18)
        self.label_description.setAlignment(Qt.AlignVCenter)        
        self.label_description.setText(self.user_description)
        self.label_description.setStyleSheet("color: #A6A6A6; font: 9pt 'Segoe UI';")

    # PAINT USER IMAGE EVENTS
    # ///////////////////////////////////////////////////////////////
    def paintEvent(self, event):
        # PAINTER USER IMAGE
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)

        # RECT
        rect = QRect(10, 15, 40, 40)

        # CIRCLE
        painter.setPen(Qt.NoPen)
        painter.setBrush(QBrush(QColor("#000000")))
        painter.drawEllipse(rect)

        # DRAW USER IMAGE
        self.draw_user_image(painter, self.user_image, rect)        

        # PAINT END
        painter.end()

        # DRAW USER IMAGE
        self.draw_status(self.user_image, rect)

    # DRAW USER IMAGE
    # ///////////////////////////////////////////////////////////////
    def draw_user_image(self, qp, image, rect):
        user_image = QImage(image)
        path = QPainterPath()
        path.addEllipse(rect)
        qp.setClipPath(path)
        qp.drawImage(rect, user_image)

    # DRAW STATUS
    # ///////////////////////////////////////////////////////////////
    def draw_status(self, status, rect):
        painter = QPainter()
        painter.begin(self)
        painter.setRenderHint(QPainter.Antialiasing)

        # PEN
        pen = QPen()
        pen.setWidth(3)
        pen.setColor(QColor("#151617"))
        painter.setPen(pen)

        # BRUSH/STATUS COLOR
        painter.setBrush(QBrush(QColor(self._status_color)))

        # DRAW
        painter.drawEllipse(rect.x() + 27, rect.y() + 27, 13, 13)
        painter.end()

# SET STYLE TO POPUP
# ///////////////////////////////////////////////////////////////
style = """
/* QFrame */
QFrame {
    background: #333436; border-radius: 10px;
}
/* Search Message */
.QLineEdit {
	border: 2px solid rgb(47, 48, 50);
	border-radius: 15px;
	background-color: rgb(47, 48, 50);
	color: rgb(121, 121, 121);
	padding-left: 10px;
	padding-right: 10px;
}
.QLineEdit:hover {
	color: rgb(230, 230, 230);
	border: 2px solid rgb(62, 63, 66);
}
.QLineEdit:focus {
	color: rgb(230, 230, 230);
	border: 2px solid rgb(53, 54, 56);
	background-color: rgb(14, 14, 15);
}
/* QPushButton */
.QPushButton{
    background-color: transparent;
    border: none;
    border-radius: 10px;
    background-repeat: no-repeat;
    background-position: left center;
    text-align: left;
    color: #999999;
    padding-left: 38px;
}
.QPushButton:hover{
    background-color: #151617;
    color: #CCCCCC;
}

"""
# CHAN STATUS POPUP
# # ///////////////////////////////////////////////////////////////
class _ChangeStatus(QFrame):
    status = Signal(str)
    def __init__(self, parent):
        QFrame.__init__(self)

        # SETUP
        # ///////////////////////////////////////////////////////////////
        self.setFixedSize(230, 205)
        self.setStyleSheet(style)
        self.setParent(parent)

        # LAYOUT AND BORDER
        # ///////////////////////////////////////////////////////////////
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(10,10,10,10)
        self.border = QFrame(self)
        self.layout.addWidget(self.border)

        # LINEEDIT AND BTNS BOX
        # ///////////////////////////////////////////////////////////////
        self.layout_content = QVBoxLayout(self.border)
        self.layout_content.setContentsMargins(0,0,0,0)
        self.layout_content.setSpacing(1)

        # CHANGE DESCRIPTION
        # ///////////////////////////////////////////////////////////////
        self.line_edit = QLineEdit()
        self.line_edit.setMinimumHeight(30)
        self.line_edit.setPlaceholderText("Write what you are doing...")

        # TOP LABEL
        # ///////////////////////////////////////////////////////////////
        self.label = QLabel("Change status:")
        self.label.setStyleSheet("padding-top: 5px; padding-bottom: 5px; color: rgb(121, 121, 121);")

        # BTN ONLINE
        # ///////////////////////////////////////////////////////////////
        self.btn_online = QPushButton()
        self.btn_online.setText("Online")
        self.btn_online.setMinimumHeight(30)
        self.btn_online.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_online.svg)")
        self.btn_online.clicked.connect(lambda: self.send_signal("online"))
        self.btn_online.setCursor(Qt.PointingHandCursor)

        # BTNL ILDE
        # ///////////////////////////////////////////////////////////////
        self.btn_idle = QPushButton()
        self.btn_idle.setText("Idle")
        self.btn_idle.setMinimumHeight(30)
        self.btn_idle.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_idle.svg)")
        self.btn_idle.clicked.connect(lambda: self.send_signal("idle"))
        self.btn_idle.setCursor(Qt.PointingHandCursor)

        # BTN BUSE
        # ///////////////////////////////////////////////////////////////
        self.btn_busy = QPushButton()
        self.btn_busy.setText("Do not disturb")
        self.btn_busy.setMinimumHeight(30)
        self.btn_busy.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_busy.svg)")
        self.btn_busy.clicked.connect(lambda: self.send_signal("busy"))
        self.btn_busy.setCursor(Qt.PointingHandCursor)

        # BTN INVISIBLE
        self.btn_invisible = QPushButton()
        self.btn_invisible.setText("Invisible")
        self.btn_invisible.setMinimumHeight(30)
        self.btn_invisible.setStyleSheet("background-image: url(:/icons_svg/images/icons_svg/icon_invisible.svg)")
        self.btn_invisible.clicked.connect(lambda: self.send_signal("invisible"))
        self.btn_invisible.setCursor(Qt.PointingHandCursor)

        # ADD WIDGETS TO LAYOUT
        # ///////////////////////////////////////////////////////////////
        self.layout_content.addWidget(self.line_edit)
        self.layout_content.addWidget(self.label)
        self.layout_content.addWidget(self.btn_online)
        self.layout_content.addWidget(self.btn_idle)
        self.layout_content.addWidget(self.btn_busy)
        self.layout_content.addWidget(self.btn_invisible)

        # SET DROP SHADOW
        # ///////////////////////////////////////////////////////////////
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(15)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 160))
        self.setGraphicsEffect(self.shadow)
    
    # SEND SIGNAL TO TOP USER WIDGET
    # ///////////////////////////////////////////////////////////////
    def send_signal(self, status):
        self.status.emit(status)


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'pyblackbox.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from mplwidget import MplWidget
from consolewidget import ConsoleWidget

import resources_rc

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1200, 907)
        self.stylesheet = QWidget(MainWindow)
        self.stylesheet.setObjectName(u"stylesheet")
        self.stylesheet.setStyleSheet(u"/* DEFAULT */\n"
"QWidget {\n"
"	font: 9pt \"Segoe UI\";\n"
"	color: rgb(230, 230, 230);\n"
"	selection-background-color: rgb(86, 115, 0);\n"
"}\n"
"/* Bg App */\n"
"#bg_app {	\n"
"	background-color: rgb(0, 0, 0);\n"
"	border: 2px solid rgb(30, 32, 33);\n"
"}\n"
"\n"
"/* Left Menu */\n"
"#left_menu {\n"
"	border-top-left-radius: 10px;\n"
"	border-bottom-left-radius: 10px;\n"
"}\n"
"\n"
"/* Logo Top */\n"
"#logo_top {\n"
"	background-image: url(:/images_svg/images/images_svg/logo_symbol_top.svg);\n"
"	background-position: center;\n"
"	background-repeat: no-repeat;\n"
"}\n"
"\n"
"/* Buttons */\n"
"#left_menu QPushButton {\n"
"	border: none;	\n"
"	background-color: transparent;\n"
"	border-radius: 10px;\n"
"	background-repeat: none;\n"
"	background-position: center;\n"
"}\n"
"#left_menu QPushButton:hover {\n"
"	background-color: rgb(21, 22, 23);\n"
"}\n"
"#left_menu QPushButton:pressed {\n"
"	background-color: rgb(172, 229, 0);\n"
"}\n"
"#add_user_btn {	\n"
"	background-image: url(:/icons_svg/images/icons_svg/ico"
                        "n_add_user.svg);\n"
"}\n"
"#settings_btn {	\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_settings.svg);\n"
"}\n"
"\n"
"/* Left Messages */\n"
"#left_messages {\n"
"	background-color: rgb(21, 22, 23);\n"
"	border: none;\n"
"	border-left: 3px solid rgb(30, 32, 33);\n"
"}\n"
"\n"
"/* Top */\n"
"#top_messages {\n"
"	border: none;\n"
"	border-bottom: 1px solid rgb(47, 48, 50);\n"
"}\n"
"\n"
"/* Search Message */\n"
"#search_sms_frame .QLineEdit {\n"
"	border: 2px solid rgb(47, 48, 50);\n"
"	border-radius: 15px;\n"
"	background-color: rgb(47, 48, 50);\n"
"	color: rgb(121, 121, 121);\n"
"	padding-left: 30px;\n"
"	padding-right: 10px;\n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_search.svg);\n"
"	background-repeat: none;\n"
"	background-position: left center;\n"
"}\n"
"#search_sms_frame .QLineEdit:hover {\n"
"	color: rgb(230, 230, 230);\n"
"	border: 2px solid rgb(62, 63, 66);\n"
"}\n"
"#search_sms_frame .QLineEdit:focus {\n"
"	color: rgb(230, 230, 230);\n"
"	border: 2px solid rgb(53, 54"
                        ", 56);\n"
"	background-color: rgb(14, 14, 15);\n"
"}\n"
"\n"
"/* Menus Scroll Area */\n"
"#left_messages_scroll, #messages_scroll {\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"/* Bottom / Signal */\n"
"#bottom_messages {	\n"
"	background-color: rgb(30, 32, 33);\n"
"}\n"
"#signal_icon { \n"
"	background-image: url(:/icons_svg/images/icons_svg/icon_signal.svg);\n"
"	background-repeat: none;\n"
"	background-position: left center;\n"
"}\n"
"#label_top{	\n"
"	font: 800 10pt \"Segoe UI\";\n"
"	color: rgb(189, 255, 0);\n"
"}\n"
"#label_bottom {	\n"
"	color: rgb(166, 166, 166);\n"
"}\n"
"\n"
"/* Right Content */\n"
"#right_content {\n"
"	border-top-right-radius: 10px;\n"
"	border-bottom-right-radius: 10px;\n"
"}\n"
"\n"
"/* Top Bar */\n"
"#top_bar {\n"
"	border-top-right-radius: 10px;\n"
"}\n"
"\n"
"/* Title Bar */\n"
"#title_bar {\n"
"	background-image: url(:/images_svg/images/images_svg/text_logo.svg);\n"
"	background-repeat: no-repeat;\n"
"	background-position: left center;\n"
"	border-left: 15px solid trans"
                        "parent;\n"
"}\n"
"\n"
"/* Top BTNs */\n"
"#top_btns {  }\n"
"#top_btns .QPushButton {	\n"
"	background-position: center;\n"
"    background-repeat: no-repeat;\n"
"	border: none;\n"
"	outline: none;\n"
"	border-radius: 8px;\n"
"	text-align: left;\n"
"}\n"
"#top_btns .QPushButton:hover { background-color: rgb(21, 22, 23); }\n"
"#top_btns .QPushButton:pressed { background-color: rgb(172, 229, 0); }\n"
"#top_btns #close_app_btn:hover { background-color: rgb(255, 0, 127); }\n"
"#top_btns #close_app_btn:pressed { background-color: rgb(172, 229, 0); }\n"
"\n"
"/* Content / Pages */\n"
"#app_pages {\n"
"	background-color: transparent;\n"
"}\n"
"/* /////////////////////////////////////////////////////////////////////////////////////////////////\n"
"ScrollBars */\n"
"QScrollBar:horizontal {\n"
"    border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    height: 8px;\n"
"    margin: 0px 21px 0 21px;\n"
"	border-radius: 0px;\n"
"}\n"
"QScrollBar::handle:horizontal {\n"
"    background: rgb(47, 48, 50);\n"
"    min-widt"
                        "h: 25px;\n"
"	border-radius: 4px\n"
"}\n"
"QScrollBar::add-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-right-radius: 4px;\n"
"    border-bottom-right-radius: 4px;\n"
"    subcontrol-position: right;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QScrollBar::sub-line:horizontal {\n"
"    border: none;\n"
"    background: rgb(55, 63, 77);\n"
"    width: 20px;\n"
"	border-top-left-radius: 4px;\n"
"    border-bottom-left-radius: 4px;\n"
"    subcontrol-position: left;\n"
"    subcontrol-origin: margin;\n"
"}\n"
"QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
"QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal\n"
"{\n"
"     background: none;\n"
"}\n"
" QScrollBar:vertical {\n"
"	border: none;\n"
"    background: rgb(52, 59, 72);\n"
"    width: 8px;\n"
"    margin: 21px 0 21px 0;\n"
"	border-radius: 0px;\n"
" }\n"
" QScrollBar::handle:vertical {	\n"
"	background: rgb(47, 48"
                        ", 50);\n"
"    min-height: 25px;\n"
"	border-radius: 4px\n"
" }\n"
" QScrollBar::add-line:vertical {\n"
"     border: none;\n"
"    background: transparent;\n"
"     height: 10px;\n"
"    border-radius: 4px;\n"
"     subcontrol-position: bottom;\n"
"     subcontrol-origin: margin;\n"
" }\n"
" QScrollBar::sub-line:vertical {\n"
"	border: none;\n"
"    background: transparent;\n"
"    height: 10px;\n"
"    border-radius: 4px;\n"
"     subcontrol-position: top;\n"
"     subcontrol-origin: margin;\n"
" }\n"
" QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
" QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {\n"
"     background: none;\n"
" }\n"
"\n"
"")
        self.margins_app = QVBoxLayout(self.stylesheet)
        self.margins_app.setSpacing(0)
        self.margins_app.setObjectName(u"margins_app")
        self.margins_app.setContentsMargins(10, 10, 10, 10)
        self.bg_app = QFrame(self.stylesheet)
        self.bg_app.setObjectName(u"bg_app")
        self.bg_app.setStyleSheet(u"#bg_app { border-radius: 10px; }")
        self.bg_app.setFrameShape(QFrame.NoFrame)
        self.bg_app.setFrameShadow(QFrame.Raised)
        self.bg_app.setLineWidth(0)
        self.base_Layout = QVBoxLayout(self.bg_app)
        self.base_Layout.setSpacing(0)
        self.base_Layout.setObjectName(u"base_Layout")
        self.base_Layout.setContentsMargins(0, 0, 0, 0)
        self.horizontal_Layout = QHBoxLayout()
        self.horizontal_Layout.setSpacing(0)
        self.horizontal_Layout.setObjectName(u"horizontal_Layout")
        self.left_menu = QFrame(self.bg_app)
        self.left_menu.setObjectName(u"left_menu")
        self.left_menu.setMinimumSize(QSize(50, 0))
        self.left_menu.setMaximumSize(QSize(50, 16777215))
        self.left_menu.setFrameShape(QFrame.NoFrame)
        self.left_menu.setFrameShadow(QFrame.Raised)
        self.left_menu.setLineWidth(0)
        self.vertical_left_menu_layout = QVBoxLayout(self.left_menu)
        self.vertical_left_menu_layout.setSpacing(0)
        self.vertical_left_menu_layout.setObjectName(u"vertical_left_menu_layout")
        self.vertical_left_menu_layout.setContentsMargins(0, 0, 0, 0)
        self.logo_top = QLabel(self.left_menu)
        self.logo_top.setObjectName(u"logo_top")
        self.logo_top.setMinimumSize(QSize(50, 50))
        self.logo_top.setMaximumSize(QSize(50, 50))

        self.vertical_left_menu_layout.addWidget(self.logo_top)

        self.top_menus = QFrame(self.left_menu)
        self.top_menus.setObjectName(u"top_menus")
        self.top_menus.setMinimumSize(QSize(0, 50))
        self.top_menus.setFrameShape(QFrame.NoFrame)
        self.top_menus.setFrameShadow(QFrame.Raised)
        self.top_menus_layout = QVBoxLayout(self.top_menus)
        self.top_menus_layout.setSpacing(5)
        self.top_menus_layout.setObjectName(u"top_menus_layout")
        self.top_menus_layout.setContentsMargins(5, 5, 5, 5)

        self.vertical_left_menu_layout.addWidget(self.top_menus)

        self.spacer_vertical_menu = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.vertical_left_menu_layout.addItem(self.spacer_vertical_menu)

        self.bottom_menus = QFrame(self.left_menu)
        self.bottom_menus.setObjectName(u"bottom_menus")
        self.bottom_menus.setMinimumSize(QSize(0, 50))
        self.bottom_menus.setFrameShape(QFrame.NoFrame)
        self.bottom_menus.setFrameShadow(QFrame.Raised)
        self.bottom_menus_layout = QVBoxLayout(self.bottom_menus)
        self.bottom_menus_layout.setSpacing(5)
        self.bottom_menus_layout.setObjectName(u"bottom_menus_layout")
        self.bottom_menus_layout.setSizeConstraint(QLayout.SetMinAndMaxSize)
        self.bottom_menus_layout.setContentsMargins(5, 5, 5, 5)

        self.vertical_left_menu_layout.addWidget(self.bottom_menus)


        self.horizontal_Layout.addWidget(self.left_menu)

        self.left_messages = QFrame(self.bg_app)
        self.left_messages.setObjectName(u"left_messages")
        self.left_messages.setMinimumSize(QSize(243, 0))
        self.left_messages.setMaximumSize(QSize(243, 16777215))
        self.left_messages.setFrameShape(QFrame.NoFrame)
        self.left_messages.setFrameShadow(QFrame.Raised)
        self.left_messages.setLineWidth(0)
        self.left_box_layout = QVBoxLayout(self.left_messages)
        self.left_box_layout.setSpacing(0)
        self.left_box_layout.setObjectName(u"left_box_layout")
        self.left_box_layout.setContentsMargins(0, 0, 0, 0)
        self.top_messages = QFrame(self.left_messages)
        self.top_messages.setObjectName(u"top_messages")
        self.top_messages.setMinimumSize(QSize(0, 105))
        self.top_messages.setMaximumSize(QSize(16777215, 105))
        self.top_messages.setFrameShape(QFrame.NoFrame)
        self.top_messages.setFrameShadow(QFrame.Raised)
        self.top_messages_layout = QVBoxLayout(self.top_messages)
        self.top_messages_layout.setSpacing(0)
        self.top_messages_layout.setObjectName(u"top_messages_layout")
        self.top_messages_layout.setContentsMargins(0, 0, 0, 0)
        self.top_user_frame = QFrame(self.top_messages)
        self.top_user_frame.setObjectName(u"top_user_frame")
        self.top_user_frame.setMinimumSize(QSize(0, 60))
        self.top_user_frame.setMaximumSize(QSize(16777215, 60))
        self.top_user_frame.setFrameShape(QFrame.NoFrame)
        self.top_user_frame.setFrameShadow(QFrame.Raised)

        self.top_messages_layout.addWidget(self.top_user_frame)

        self.search_sms_frame = QFrame(self.top_messages)
        self.search_sms_frame.setObjectName(u"search_sms_frame")
        self.search_sms_frame.setMinimumSize(QSize(0, 40))
        self.search_sms_frame.setMaximumSize(QSize(16777215, 40))
        self.search_sms_frame.setFrameShape(QFrame.NoFrame)
        self.search_sms_frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_6 = QVBoxLayout(self.search_sms_frame)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(10, 0, 10, 0)
        self.search_line_edit = QLineEdit(self.search_sms_frame)
        self.search_line_edit.setObjectName(u"search_line_edit")
        self.search_line_edit.setMinimumSize(QSize(0, 30))
        self.search_line_edit.setMaximumSize(QSize(16777215, 40))

        self.verticalLayout_6.addWidget(self.search_line_edit, 0, Qt.AlignVCenter)


        self.top_messages_layout.addWidget(self.search_sms_frame)


        self.left_box_layout.addWidget(self.top_messages)

        self.left_messages_scroll = QScrollArea(self.left_messages)
        self.left_messages_scroll.setObjectName(u"left_messages_scroll")
        self.left_messages_scroll.setFrameShape(QFrame.NoFrame)
        self.left_messages_scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.left_messages_scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.left_messages_scroll.setWidgetResizable(True)
        self.messages_scroll = QWidget()
        self.messages_scroll.setObjectName(u"messages_scroll")
        self.messages_scroll.setGeometry(QRect(0, 0, 240, 711))
        self.messages_layout_base = QVBoxLayout(self.messages_scroll)
        self.messages_layout_base.setSpacing(0)
        self.messages_layout_base.setObjectName(u"messages_layout_base")
        self.messages_layout_base.setContentsMargins(0, 5, 0, 5)
        self.messages_frame = QFrame(self.messages_scroll)
        self.messages_frame.setObjectName(u"messages_frame")
        self.messages_frame.setFrameShape(QFrame.NoFrame)
        self.messages_frame.setFrameShadow(QFrame.Raised)
        self.messages_layout = QVBoxLayout(self.messages_frame)
        self.messages_layout.setSpacing(5)
        self.messages_layout.setObjectName(u"messages_layout")
        self.messages_layout.setContentsMargins(0, 0, 0, 0)

        self.messages_layout_base.addWidget(self.messages_frame, 0, Qt.AlignTop)

        self.left_messages_scroll.setWidget(self.messages_scroll)

        self.left_box_layout.addWidget(self.left_messages_scroll)

        self.bottom_messages = QFrame(self.left_messages)
        self.bottom_messages.setObjectName(u"bottom_messages")
        self.bottom_messages.setMinimumSize(QSize(0, 65))
        self.bottom_messages.setMaximumSize(QSize(16777215, 65))
        self.bottom_messages.setFrameShape(QFrame.NoFrame)
        self.bottom_messages.setFrameShadow(QFrame.Raised)
        self.verticalLayout_5 = QVBoxLayout(self.bottom_messages)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(15, 0, 10, 0)
        self.signal_frame = QFrame(self.bottom_messages)
        self.signal_frame.setObjectName(u"signal_frame")
        self.signal_frame.setMinimumSize(QSize(0, 35))
        self.signal_frame.setMaximumSize(QSize(16777215, 35))
        self.signal_frame.setFrameShape(QFrame.NoFrame)
        self.signal_frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.signal_frame)
        self.horizontalLayout_2.setSpacing(10)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.signal_icon = QFrame(self.signal_frame)
        self.signal_icon.setObjectName(u"signal_icon")
        self.signal_icon.setMaximumSize(QSize(30, 16777215))
        self.signal_icon.setFrameShape(QFrame.NoFrame)
        self.signal_icon.setFrameShadow(QFrame.Raised)

        self.horizontalLayout_2.addWidget(self.signal_icon)

        self.signal_text = QFrame(self.signal_frame)
        self.signal_text.setObjectName(u"signal_text")
        self.signal_text.setFrameShape(QFrame.NoFrame)
        self.signal_text.setFrameShadow(QFrame.Raised)
        self.verticalLayout_7 = QVBoxLayout(self.signal_text)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.verticalLayout_7.setContentsMargins(0, 2, 0, 2)
        self.label_top = QLabel(self.signal_text)
        self.label_top.setObjectName(u"label_top")

        self.verticalLayout_7.addWidget(self.label_top)

        self.label_bottom = QLabel(self.signal_text)
        self.label_bottom.setObjectName(u"label_bottom")

        self.verticalLayout_7.addWidget(self.label_bottom)


        self.horizontalLayout_2.addWidget(self.signal_text)


        self.verticalLayout_5.addWidget(self.signal_frame, 0, Qt.AlignVCenter)


        self.left_box_layout.addWidget(self.bottom_messages)


        self.horizontal_Layout.addWidget(self.left_messages)

        self.right_content = QFrame(self.bg_app)
        self.right_content.setObjectName(u"right_content")
        font = QFont()
        font.setFamily(u"Segoe UI")
        font.setPointSize(9)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        font.setStyleStrategy(QFont.PreferAntialias)
        self.right_content.setFont(font)
        self.right_content.setFrameShape(QFrame.NoFrame)
        self.right_content.setFrameShadow(QFrame.Raised)
        self.right_content.setLineWidth(0)
        self.verticalLayout = QVBoxLayout(self.right_content)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.top_bar = QFrame(self.right_content)
        self.top_bar.setObjectName(u"top_bar")
        self.top_bar.setMinimumSize(QSize(0, 45))
        self.top_bar.setMaximumSize(QSize(16777215, 45))
        self.top_bar.setFrameShape(QFrame.NoFrame)
        self.top_bar.setFrameShadow(QFrame.Raised)
        self.top_bar.setLineWidth(0)
        self.horizontalLayout = QHBoxLayout(self.top_bar)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.title_bar = QLabel(self.top_bar)
        self.title_bar.setObjectName(u"title_bar")
        self.title_bar.setLineWidth(0)

        self.horizontalLayout.addWidget(self.title_bar)

        self.top_btns = QFrame(self.top_bar)
        self.top_btns.setObjectName(u"top_btns")
        self.top_btns.setMaximumSize(QSize(100, 16777215))
        self.top_btns.setFrameShape(QFrame.NoFrame)
        self.top_btns.setFrameShadow(QFrame.Raised)
        self.top_btns.setLineWidth(0)
        self.top_btn_layout = QHBoxLayout(self.top_btns)
        self.top_btn_layout.setSpacing(4)
        self.top_btn_layout.setObjectName(u"top_btn_layout")
        self.top_btn_layout.setContentsMargins(0, 0, 0, 0)
        self.minimize_app_btn = QPushButton(self.top_btns)
        self.minimize_app_btn.setObjectName(u"minimize_app_btn")
        self.minimize_app_btn.setMinimumSize(QSize(28, 28))
        self.minimize_app_btn.setMaximumSize(QSize(28, 28))
        font1 = QFont()
        font1.setFamily(u"Segoe UI")
        font1.setPointSize(9)
        font1.setBold(False)
        font1.setItalic(False)
        font1.setWeight(50)
        font1.setStyleStrategy(QFont.NoAntialias)
        self.minimize_app_btn.setFont(font1)
        self.minimize_app_btn.setCursor(QCursor(Qt.PointingHandCursor))
        self.minimize_app_btn.setStyleSheet(u"background-image: url(:/icons_svg/images/icons_svg/icon_minimize.svg);")
        self.minimize_app_btn.setIconSize(QSize(20, 20))

        self.top_btn_layout.addWidget(self.minimize_app_btn)

        self.maximize_restore_app_btn = QPushButton(self.top_btns)
        self.maximize_restore_app_btn.setObjectName(u"maximize_restore_app_btn")
        self.maximize_restore_app_btn.setMinimumSize(QSize(28, 28))
        self.maximize_restore_app_btn.setMaximumSize(QSize(28, 28))
        font2 = QFont()
        font2.setFamily(u"Segoe UI")
        font2.setPointSize(9)
        font2.setBold(False)
        font2.setItalic(False)
        font2.setWeight(50)
        font2.setStyleStrategy(QFont.PreferDefault)
        self.maximize_restore_app_btn.setFont(font2)
        self.maximize_restore_app_btn.setCursor(QCursor(Qt.PointingHandCursor))
        self.maximize_restore_app_btn.setStyleSheet(u"background-image: url(:/icons_svg/images/icons_svg/icon_maximize.svg);")
        self.maximize_restore_app_btn.setIconSize(QSize(20, 20))

        self.top_btn_layout.addWidget(self.maximize_restore_app_btn)

        self.close_app_btn = QPushButton(self.top_btns)
        self.close_app_btn.setObjectName(u"close_app_btn")
        self.close_app_btn.setMinimumSize(QSize(28, 28))
        self.close_app_btn.setMaximumSize(QSize(28, 28))
        self.close_app_btn.setCursor(QCursor(Qt.PointingHandCursor))
        self.close_app_btn.setStyleSheet(u"background-image: url(:/icons_svg/images/icons_svg/icon_close.svg);")
        self.close_app_btn.setIconSize(QSize(20, 20))

        self.top_btn_layout.addWidget(self.close_app_btn)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.top_btn_layout.addItem(self.horizontalSpacer)


        self.horizontalLayout.addWidget(self.top_btns)


        self.verticalLayout.addWidget(self.top_bar)

        self.content = QFrame(self.right_content)
        self.content.setObjectName(u"content")
        self.content.setFrameShape(QFrame.NoFrame)
        self.content.setFrameShadow(QFrame.Raised)
        self.content.setLineWidth(0)
        self.verticalLayout_4 = QVBoxLayout(self.content)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.app_pages = QStackedWidget(self.content)
        self.app_pages.setObjectName(u"app_pages")
        self.app_pages.setStyleSheet(u"background-color: transparent;")
        self.home = QWidget()
        self.home.setObjectName(u"home")
        self.home.setStyleSheet(u"#home {\n"
"	background-position: center;\n"
"	background-repeat: no-repeat;\n"
"	background-image: url(:/images_svg/images/images_svg/logo_home.svg);\n"
"}")
        self.app_pages.addWidget(self.home)
        self.chat = QWidget()
        self.chat.setObjectName(u"chat")
        self.chat_layout = QVBoxLayout(self.chat)
        self.chat_layout.setSpacing(0)
        self.chat_layout.setObjectName(u"chat_layout")
        self.chat_layout.setContentsMargins(0, 0, 0, 0)
        self.app_pages.addWidget(self.chat)
        self.page_scope = QWidget()
        self.page_scope.setObjectName(u"page_scope")
        self.verticalLayout_2 = QVBoxLayout(self.page_scope)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.PBB_MplWidget_ACMPlot = MplWidget(self.page_scope)
        self.PBB_MplWidget_ACMPlot.setObjectName(u"PBB_MplWidget_ACMPlot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.PBB_MplWidget_ACMPlot.sizePolicy().hasHeightForWidth())
        self.PBB_MplWidget_ACMPlot.setSizePolicy(sizePolicy)
        self.PBB_MplWidget_ACMPlot.setMinimumSize(QSize(800, 800))
        self.PBB_MplWidget_ACMPlot.setStyleSheet(u"background-color: rgb(170, 0, 127)")

        self.verticalLayout_2.addWidget(self.PBB_MplWidget_ACMPlot)

        self.app_pages.addWidget(self.page_scope)
        self.page_console = QWidget()
        self.page_console.setObjectName(u"page_console")
        self.verticalLayout_3 = QVBoxLayout(self.page_console)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.PBB_ConsoleWidget = ConsoleWidget(self.page_console)
        self.PBB_ConsoleWidget.setObjectName(u"PBB_ConsoleWidget")
        self.PBB_ConsoleWidget.setEnabled(True)
        self.PBB_ConsoleWidget.setMinimumSize(QSize(500, 300))
        self.PBB_ConsoleWidget.setStyleSheet(u"background-color: rgb(85, 0, 127);")

        self.verticalLayout_3.addWidget(self.PBB_ConsoleWidget)

        self.app_pages.addWidget(self.page_console)

        self.verticalLayout_4.addWidget(self.app_pages)


        self.verticalLayout.addWidget(self.content)


        self.horizontal_Layout.addWidget(self.right_content)


        self.base_Layout.addLayout(self.horizontal_Layout)


        self.margins_app.addWidget(self.bg_app)

        MainWindow.setCentralWidget(self.stylesheet)

        self.retranslateUi(MainWindow)

        self.app_pages.setCurrentIndex(3)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.search_line_edit.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Search messages", None))
        self.label_top.setText(QCoreApplication.translate("MainWindow", u"Signal 80%", None))
        self.label_bottom.setText(QCoreApplication.translate("MainWindow", u"PyBlackBOX server signal", None))
        self.title_bar.setText("")
#if QT_CONFIG(tooltip)
        self.minimize_app_btn.setToolTip(QCoreApplication.translate("MainWindow", u"Minimize", None))
#endif // QT_CONFIG(tooltip)
        self.minimize_app_btn.setText("")
#if QT_CONFIG(tooltip)
        self.maximize_restore_app_btn.setToolTip(QCoreApplication.translate("MainWindow", u"Maximize", None))
#endif // QT_CONFIG(tooltip)
        self.maximize_restore_app_btn.setText("")
#if QT_CONFIG(tooltip)
        self.close_app_btn.setToolTip(QCoreApplication.translate("MainWindow", u"Close", None))
#endif // QT_CONFIG(tooltip)
        self.close_app_btn.setText("")
    # retranslateUi




from app_modules import *
if bool_use_PyQt5:
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtCore import QFile, QTextStream
else:
    from PySide2.QtWidgets import QApplication
    from PySide2.QtCore import QFile, QTextStream

import pkg_resources # to include resource file: mainWindow.ui # https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package/20885799

def toggle_stylesheet(path):
    '''
    Toggle the stylesheet to use the desired path in the Qt resource
    system (prefixed by `:/`) or generically (a path to a file on
    system).

    :path:      A full path to a resource or file on system
    '''

    filepath_to_qssFile = pkg_resources.resource_filename(__name__, path)

    # get the QApplication instance,  or crash if not set
    app = QApplication.instance()
    if app is None:
        raise RuntimeError("No Qt Application found.")

    if filepath_to_qssFile == "":
        app.setStyleSheet("")
        return

    file = QFile(filepath_to_qssFile)
    file.open(QFile.ReadOnly | QFile.Text)
    stream = QTextStream(file)
    app.setStyleSheet(stream.readAll())



import os, sys; sys.path.insert(1, '../../CJHAcademicPlot'); 

from __cjhAcademicPlotSettings import CJHStylePlot

if __name__ == '__main__':
    csp = CJHStylePlot(path2info_dot_dat=__file__, nrows=6)

    list_of_ylabel = [
      'Speed\n[rpm]',
      '$\\alpha$-axis Flux\n[Wb]',
      'Torque Current\n[A]',
      'Offset Voltage\n[V]',
      'Magn. Current\n[A]',
      'Load Torque\n[Nm]',
    ]

    list_of_signalNames = [
    ######################################
    ("ACM.rpm_cmd",
    "ACM.rpm",
    "marino.xOmg*ELEC_RAD_PER_SEC_2_RPM",
    ),
    ######################################
    ("ACM.psi_Amu",
    "ohtani.psi_2_real_output[0]",
    ),
    ######################################
    ("CTRL.I->iDQ_cmd[1]",
    # "ACM.iTs",
    "CTRL.I->iDQ[1]",
    ) ,
    ######################################
    ("ohtani.correction_integral_term[0]",
    "ohtani.correction_integral_term[1]",
    ) ,
    ######################################
    ("CTRL.I->iDQ_cmd[0]",
    # "ACM.iMs",
    "CTRL.I->iDQ[0]",
    ) ,
    ######################################
    ("ACM.TLoad",
    "marino.xTL",
    ),
    ]

    csp.plot(list_of_ylabel, list_of_signalNames)

# Obsolete 
if __name__ == '!__main__':
    from __cjhAcademicPlotSettings import *

    df, time, fname = load_dataframe(os.path.dirname(os.path.abspath(__file__)))
    fig, axes_v = plt.subplots(ncols=1, nrows=6, sharex=True, figsize=(8,9), facecolor='w', edgecolor='k'); # modifying dpi would change the size of fonts
    # fig.subplots_adjust(wspace=0.01)

    i =0; ax = axes_v[i]; plot_it(ax, time, 'Speed\n[rpm]', OD([ 
               (r'$\omega^*$',             df["ACM.rpm_cmd"]),
               (r'$\omega$',               df["ACM.rpm"]),
               (r'$\hat\omega$',           df["marino.xOmg*ELEC_RAD_PER_SEC_2_RPM"]),
             ]))      
    i+=1; ax = axes_v[i]; plot_it(ax, time, '$\\alpha$-axis Flux\n[Wb]', OD([ 
               (r'$T_L$',                  df["ACM.psi_Amu"]),
               (r'$\hat T_L$',             df["ohtani.psi_2_real_output[0]"]),
             ]))
    i+=1; ax = axes_v[i]; plot_it(ax, time, 'Torque Current\n[A]', OD([ 
               (r'1',             df["CTRL.I->iDQ_cmd[1]"]),
               # (r'2',             df["ACM.iTs"]),
               (r'3',             df["CTRL.I->iDQ[1]"]),
             ]))
    i+=1; ax = axes_v[i]; plot_it(ax, time, 'Offset Voltage\n[V]', OD([ 
               (r'$\hat u_{\alpha,\mathrm{offset}}$', df["ohtani.correction_integral_term[0]"]),
               (r'$\hat u_{\beta,\mathrm{offset}}$',  df["ohtani.correction_integral_term[1]"]),
             ]))
    i+=1; ax = axes_v[i]; plot_it(ax, time, 'Magn. Current\n[A]', OD([ 
               (r'1',             df["CTRL.I->iDQ_cmd[0]"]),
               # (r'2',             df["ACM.iMs"]),
               (r'3',             df["CTRL.I->iDQ[0]"]),
             ]))
    i+=1; ax = axes_v[i]; plot_it(ax, time, 'Load Torque\n[Nm]', OD([ 
               (r'$T_L$',                  df["ACM.TLoad"]),
               (r'$\hat T_L$',             df["marino.xTL"]),
             ]))

    for ax in axes_v:
        ax.grid(True)
        # ax.axvspan(14, 29, facecolor='r', alpha=0.1)
        ax.set_yticklabels([f'{el:g}' for el in ax.get_yticks()], font) # https://matplotlib.org/3.3.3/api/_as_gen/matplotlib.axes.Axes.set_yticklabels.html

    # axes_v[-1].set_xlim((0,130))
    # axes_v[-1].set_xticks(np.arange(0, 201, 20))
    axes_v[-1].set_xlabel('Time, $t$ [s]', fontdict=font)
    axes_v[-1].set_xticklabels(axes_v[-1].get_xticks(), font)

    # fig.tight_layout() # tight layout is bad if you have too many subplots
    fig.savefig(              f'{fname[:-4]}.pdf', dpi=300, bbox_inches='tight', pad_inches=0)
    os.system('sumatrapdf ' + f'{fname[:-4]}.pdf')
    # plt.show()

